/*
 * Created on 2004/06/29
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * 
 * This class is always subject to changes.
 */
package jp.co.epson.ide.eclipse.gnu33;

import java.io.File;
import java.net.URL;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import jp.co.epson.ide.eclipse.gnu33.cpuinfo.CpuSettingsManager;

import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.osgi.service.datalocation.Location;
import org.eclipse.ui.IStartup;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Constants;
import org.osgi.framework.Version;


/**
 * The main plugin class to be used for Gnu33 Plugin.
 * @author D.Fujimoto
 */
public class Gnu33CorePlugin extends AbstractUIPlugin implements IStartup {

	// debug flag
	public static boolean isDebug;
	
	// plugin id must be equal to that of plugin.xml
	private static final String PLUGIN_ID = "jp.co.epson.ide.eclipse.gnu33"; //$NON-NLS-1$
	private static final String RESOURCE_BUNDLE_NAME = PLUGIN_ID + ".Gnu33CorePluginResources"; //$NON-NLS-1$ 
	
	public static final String PRODUCT_ID = PLUGIN_ID + ".product"; //$NON-NLS-1$
	public static final String PROJECT_WIZARD_ID = PLUGIN_ID + ".ui.wizards.NewGnu33CDTProjectWizard" ; //$NON-NLS-1$
	public static final String PERSPECTIVE_ID = PLUGIN_ID + ".ui.perspective"; //$NON-NLS-1$
	
	
	// default status code(for IStatus)
	public static final int STATUS_CODE_DEFAULT = -1;
	
	/**
	 * The shared instance. Provides the plug-in class to other classes contained 
	 * in this plug-in.
	 */ 
	private static Gnu33CorePlugin plugin;
	
	/**
	 * The String indicating the relative path to the tools dir from eclipse.exe
	 */
	private static String toolsRelativePath;
	
	/**
	 * The version identifier
	 */ 
	Version versionID;
	
	/**
	 * The Resource bundle used in this plug-in.
	 */ 
	private static ResourceBundle RESOURCE_BUNDLE;


	// Listeners
	private Gnu33ProjectListener pjListener;
	private SourceUnitListener srcListener;
	
	// CPU settnigs
	private CpuSettingsManager cpuManager;


	/**
	 * The constructor.
	 */
	public Gnu33CorePlugin() {
		super();
		plugin = this;
		try {
			// get the ResourceBundle
			RESOURCE_BUNDLE = ResourceBundle.getBundle(RESOURCE_BUNDLE_NAME); 
			
			// get the debug flag
			String value = Platform.getDebugOption(PLUGIN_ID + "/debug");
			if (value != null && Boolean.valueOf(value).booleanValue()) {
				isDebug = true;
			}
		   
		} catch (MissingResourceException x) {
			RESOURCE_BUNDLE = null;
			Gnu33CorePlugin.log(x);
		}

	}

	/**
	 * This method is called upon plug-in activation.
	 * @param context BundleContext
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		
		if (isDebug) {
			System.out.println(PLUGIN_ID + " Plugin started!!");
		}
		
		// get the install path
		getPlatformInstallPath();
		
		// get the toolsPathString
		// The path to gnu33 tools dir from eclipse.exe
		// No trailing path separator needed
		toolsRelativePath = getPreferenceStore().getString("gnu33tools.relativePath");
	   
		// the plugin version is accessible only after the plugin has been activated
		String version = (String) getBundle().getHeaders().get(Constants.BUNDLE_VERSION);
		versionID = new Version(version);
		
		// add the ResourceChangeListener
		pjListener = new Gnu33ProjectListener();
		getWorkspace().addResourceChangeListener(pjListener); 

		// add the ResourceChangeListner for source files
		srcListener = new SourceUnitListener();
		getWorkspace().addResourceChangeListener(
				srcListener, IResourceChangeEvent.POST_CHANGE);

		// initialize the cpu settings
		cpuManager = CpuSettingsManager.getManager();
		
	}

	/**
	 * This method is called when the plug-in is stopped.
	 * @param context BundleContext
	 */
	public void stop(BundleContext context) throws Exception {
		try {
			// remove the ResourceChangeListener
			getWorkspace().removeResourceChangeListener(pjListener);
			getWorkspace().removeResourceChangeListener(srcListener);
		} finally {
			super.stop(context);
		}

	}
	
	/**
	 * Returns the plugin's resource bundle.
	 * @return the plugin's resource bundle
	 */
	public ResourceBundle getResourceBundle() {
		return RESOURCE_BUNDLE;
	}
	
	
	/**
	 * Access to the plug-in version.
	 * @return version String
	 */
	public String getVersion() {
		
		return versionID.toString();
	}

	/**
	 * Returns the shared plug-in instance.
	 */
	public static Gnu33CorePlugin getDefault() {
		return plugin;
	}

	/**
	 * Common method for returning the basic workspace.
	 * @return the workspace
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle, or 'key' if not
	 * found.
	 * @param key the key in the resource bundle
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's install URL.
	 * @return the plugin install URL
	 */
	public static URL getInstallURL() {
		return plugin.getBundle().getEntry("/"); //$NON-NLS-1$
	}
	
	/**
	 * Returns the Eclipse Platform install path.
	 * @return install path as IPath.(e.g."c:/eclipse/")
	 */
	public static IPath getPlatformInstallPath() {
		
		Location loc = Platform.getInstallLocation();
		URL url = loc.getURL();
		File file = new File(url.getFile());
		String absPath = file.getAbsolutePath();
		IPath installPath = new Path(absPath);
		
		return installPath;
	}
	
	/**
	 * Returns an IPath(Absolute path) to the GNUxx Tools' directory.
	 * Uses relative path from eclipse.exe as a hint. 
	 * @return IPath to the GNUxx Tools' directory
	 */
	public static IPath getGnuToolsPath() {
		IPath toolsPath = getPlatformInstallPath().append(toolsRelativePath);
		return toolsPath;
	}

	/**
	 * Returns the plug-in ID.
	 * @return plug-in ID
	 */
	public static String getUniqueIdentifier() {
		if (getDefault() == null) {
			return PLUGIN_ID;
		}
		return getDefault().getBundle().getSymbolicName();
	}

   
	/**
	 * For logging.
	 * Will be written to the platform log file.
	 * @param status
	 */
	public static void log(IStatus status) {
		getDefault().getLog().log(status);
	}

	/**
	 * For logging.
	 * Will be written to the platform log file.
	 * @param message
	 */
	public static void log(String message) {
		log(new Status(IStatus.ERROR, getUniqueIdentifier(), IStatus.ERROR,
				message, null));
	}

	/**
	 * For logging.
	 * Will be written to the platform log file.
	 * @param th
	 */
	public static void log(Throwable th) {
		log(new Status(IStatus.ERROR, getUniqueIdentifier(), IStatus.ERROR,
				"Internal Error!", th)); //$NON-NLS-1$
	}

	/**
	 * This will be called just after the workbench initializes.
	 * This method has no implementation but is needed.  
	 * 
	 * @see org.eclipse.ui.IStartup#earlyStartup()
	 */
	public void earlyStartup() {
		if (isDebug) {
			System.out.println("earlyStartup");   
		}
	}

}
