/*
 * Created on 2005/07/26
 * Modified on 2006/03/10 by D.Fujimoto added updateMakeTargets() for modifying the path
 * 											to make.exe at project open(and import).
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33;

import java.util.Map;

import jp.co.epson.ide.eclipse.gnu33.build.BuildSettingFactory;
import jp.co.epson.ide.eclipse.gnu33.others.Logger;
import jp.co.epson.ide.eclipse.gnu33.variables.ToolPathVarResolver;

import org.eclipse.cdt.make.core.IMakeCommonBuildInfo;
import org.eclipse.cdt.make.core.IMakeTarget;
import org.eclipse.cdt.make.core.IMakeTargetManager;
import org.eclipse.cdt.make.core.MakeBuilder;
import org.eclipse.cdt.make.core.MakeCorePlugin;
import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

/**
 * Util class that updates the project file as a Job.
 * This class will modify the path to make.exe which resides in
 * the .project file.
 * 
 * And also, the Make Targets setting in the .cdtproject file.
 * 
 * @author D.Fujimoto
 * @since 1.1.1
 * @see jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectListener
 *
 */
public class ProjectFilePathUpdater extends WorkspaceJob {

	public static final String JOB_NAME = "project file path updater";
	
	/**
	 * The project subject to update
	 */
	private IProject project;
	
	/**
	 * The family name of this job
	 */
	protected String family;
	
	
	/**
	 * @param projectName	
	 * 			the project which linkerscript setting needs to be changed
	 */
	public ProjectFilePathUpdater(String projectName) {
		super(JOB_NAME);
		this.family = projectName;
	}
	
	/**
	 * 
	 * @param project	the target project.
	 */
	public ProjectFilePathUpdater(IProject project) {
		this(project.getName());
		this.project = project; 

		setPriority(Job.INTERACTIVE);
		setSystem(true);
		// the scheduling rule should be the workspace root
		setRule(ResourcesPlugin.getWorkspace().getRoot());
	
	}

	/* ( Javadoc)
	 * @see org.eclipse.core.resources.WorkspaceJob#runInWorkspace(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public IStatus runInWorkspace(IProgressMonitor monitor)
			throws CoreException {

		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}
		try {
			monitor.beginTask("Updating project file for " + project.getName(), 2);
			
			
			// do the update
			updateProjectFile(project);
			monitor.worked(1);
			
			// update for Make Target settings
			updateMakeTargets(project);
			monitor.worked(1);

			monitor.done();
			
		} catch (CoreException e) {
			e.printStackTrace();
			Logger.log("Failed to update " + IProjectDescription.DESCRIPTION_FILE_NAME + " for " + project.getName());
			return e.getStatus();
		}
		return Status.OK_STATUS;
	}

	
	/**
	 * 
	 * @see org.eclipse.core.runtime.jobs.Job#belongsTo(java.lang.Object)
	 */
	public boolean belongsTo(Object family) {
		return this.family.equals(family);
	}
	/**
	 * The main operation on changing the project file.
	 * Rewrites the .project file's make command
	 * 
	 * @param project
	 */
	public synchronized void updateProjectFile(IProject project) throws CoreException {
	
    	// prepare the new paths
		String makePathStr = "${" + ToolPathVarResolver.TOOL_LOC + "}/" + ResourceProvider.getResourceString("gnu33tools.make.exe");
		IPath compilerPath = ToolPathVarResolver.getGnuToolsPath()
			.append(BuildSettingFactory.COMPILER_CMD);
    	
		String makeCmdProp = IMakeCommonBuildInfo.BUILD_COMMAND;
    	// get the .project file and rewrite it
    	IProjectDescription projDesc = null;
    	
    	// read the project file 
		projDesc = project.getDescription();
		ICommand[] commands = projDesc.getBuildSpec();
		
		boolean needSave = false;
		for (int i = 0; i < commands.length; i++) {
			
			if (commands[i].getBuilderName().equals(MakeBuilder.BUILDER_ID)) {
				// the make command
				Map argMap = commands[i].getArguments();
				
				// change it only if it is different
				String cmd = (String) argMap.get(makeCmdProp);	
				if (cmd != null && !cmd.equals(makePathStr)) {
					argMap.put(makeCmdProp, makePathStr);
		            commands[i].setArguments(argMap);
					needSave |= true;
				}
				
			}
		}
		
		// save it if needed
		if (needSave) {
			projDesc.setBuildSpec(commands);
			project.setDescription(projDesc, null);
			if (Gnu33CorePlugin.isDebug) {
				System.out.println(".project update done");
			}
		}
	
	}
	
	/**
	 * The additional operation on changing the (cdt)project file.
	 * Rewrites the Make Target settings i.e the .cdtproject file's make command 
	 * @param project
	 * @since 1.3.0
	 */
	public synchronized void updateMakeTargets(IProject project) throws CoreException {
		
    	// prepare the new paths(do it here because it might be updated in other methods)
		String makePathStr = "${" + ToolPathVarResolver.TOOL_LOC + "}/" + ResourceProvider.getResourceString("gnu33tools.make.exe");
		
		// get the MakeTargetManager
		IMakeTargetManager man = MakeCorePlugin.getDefault().getTargetManager();
		
		// nothing to do if the project has no Make Target settings
		if (!man.hasTargetBuilder(project)) {
			return;
		}
		
		// get the targets
		IMakeTarget[] targets = man.getTargets(project);

		// change it only if the default is off and the path is different
		for (int i = 0; i < targets.length; i++) {
			if (targets[i].isDefaultBuildCmd() == false) {
				if (targets[i].getBuildCommand().equals(makePathStr) == false) {
					targets[i].setBuildAttribute(IMakeCommonBuildInfo.BUILD_COMMAND, makePathStr);
				}
			}
		}
		
	}
	
	
	/**
	 * The job will be scheduled if the target project exists and is open.
	 * @see org.eclipse.core.runtime.jobs.Job#shouldSchedule()
	 */
	public boolean shouldSchedule() {
		if (!project.exists()) {
			return false;
		} else if (!project.isOpen()) {
			return false;
		}
		return true;
	}
	
	/**
	 * The job will be run if the target project exists.
	 * @see org.eclipse.core.runtime.jobs.Job#shouldRun()
	 */
	public boolean shouldRun() {
		if (!project.exists() || !project.getFile(IProjectDescription.DESCRIPTION_FILE_NAME).exists()) {
			return false;
		} else if (!project.isOpen()) {
			return false;
		}
		return true;
	}

}
