/*
 * Created on 2004/09/15
 * Modified on 2005/10/11 by D.Fujimoto created setDefaultSettingName() 
 * 										modified Constuctor(String).
 * Modified on 2006/03/28 by D.Fujimoto added new methods for EnvVars and
 * 											modified createDefault().
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.build;

import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;


/**
 * This class represents the build option settings.
 * Contains IBuildTools which represent compiler/assembler/linker.
 * Settings in the descriptor file is stored here and this object is 
 * stored in Gnu33Descriptor.
 * @author D.Fujimoto
 *
 */
public class BuildSetting {

	// common suffix for file name
	private static final String SUFFIX = ResourceProvider.getResourceString("common.suffix"); 

	/**
	 * The name for this setting.
	 */
	private String settingName = "default"; //$NON-NLS-1$
	
	/**
	 * The makefile name.
	 */
	private String makefileName;
	
	/**
	 * Compiler setting.
	 */
	private IBuildTool compiler;
	/**
	 * Assembler setting.
	 */
	private IBuildTool assembler;
	/**
	 * Linker setting.
	 */
	private IBuildTool linker;
	/**
	 * Environment setting
	 */
	private List envVars;
 
	/**
	 * Switch for 2pass/1pass build
	 */
	private boolean is2passBuild;
	
	/**
	 * Enum for build goal
	 * 0 = elf
	 * 1 = psa
	 */
	private int buildGoal;
	public static final int GOAL_ELF = 0;
	public static final int GOAL_PSA = 1;	// unused
	
	
	/**
	 * Constructor.
	 * Creates a default BuildSetting. 
	 */
	public BuildSetting() {
		super();
	}
	
	/**
	 * Constructor.
	 */
	public BuildSetting(String settingName) {
		this();
		setDefaultSettingName(settingName);
	}
	
	/**
	 * Create default settings.
	 * This is intended to be called at project creation and
	 * providing defaults when reading the project file.
	 * 
	 * @see BuildSettingFactory#createCompiler(int)
	 * @see BuildSettingFactory#createAssembler(int)
	 * @see BuildSettingFactory#createLinker(String, GeneralInfo)
	 * @see BuildSettingFactory#createEnvVars()
	 */
	public void createDefault(String projectName, GeneralInfo generalInfo) {
		compiler = BuildSettingFactory.createCompiler(generalInfo.getCpuType());
		assembler = BuildSettingFactory.createAssembler(generalInfo.getCpuType());
		linker = BuildSettingFactory.createLinker(
				projectName + SUFFIX, generalInfo);
		envVars = BuildSettingFactory.createEnvVars();
		BuildSettingFactory.switchCpuOptions(this, generalInfo.getCpuType());
//		BuildSettingFactory.switchMemModelOptions(this, generalInfo.getMemoryModel());
		
		is2passBuild = true;	// default
		buildGoal = GOAL_ELF;	// default
	}
	
	/**
	 * Wrapper for creating a default setting.
	 * @param projectName	
	 * @param generalInfo	the key to determine some options 
	 * @return	default BuildSetting
	 * @since 1.4.0
	 */
	public static BuildSetting createDefaultSetting(String projectName, GeneralInfo generalInfo) {
		BuildSetting setting = new BuildSetting(projectName);
		setting.createDefault(projectName, generalInfo);
		return setting;
	}
	
	

	public List getEnvVars() {
		return envVars;
	}
	public void setEnvVars(List envVars) {
		this.envVars = envVars;
	}

	public IBuildTool getAssembler() {
		return assembler;
	}
	public void setAssembler(IBuildTool assembler) {
		this.assembler = assembler;
	}
	public IBuildTool getCompiler() {
		return compiler;
	}
	public void setCompiler(IBuildTool compiler) {
		this.compiler = compiler;
	}
	public IBuildTool getLinker() {
		return linker;
	}
	public void setLinker(IBuildTool linker) {
		this.linker = linker;
	}

	public String getMakefileName() {
		return makefileName;
	}
	public void setMakefileName(String makefileName) {
		this.makefileName = makefileName;
	}
	public String getSettingName() {
		return settingName;
	}
	public void setSettingName(String settingName) {
		this.settingName = settingName;
	}
	public boolean is2passBuild() {
		return is2passBuild;
	}
	public void setIs2passBuild(boolean is2passBuild) {
		this.is2passBuild = is2passBuild;
	}
	public int getBuildGoal() {
		return buildGoal;
	}
	public void setBuildGoal(int buildGoal) {
		this.buildGoal = buildGoal;
	}

	/**
	 * Sets the  default setting names.
	 * @param settingName	the base name for the setting.
	 * @since 1.2.0
	 */
	public void setDefaultSettingName(String settingName) {
		setSettingName(settingName);
		setMakefileName(settingName + SUFFIX);
	}

	
}
