/*
 * Created on 2004/08/26
 * Modified on 2005/08/19 by D.Fujimoto modified setPeDefaults().
 * Modified on 2005/10/11 by D.Fujimoto added setDefaultSettingName().
 * Modified on 2006/03/13 by D.Fujimoto added rawCommands for user commands.
 * Modified on 2006/03/22 by D.Fujimoto added new Constructor.
 * Modified on 2006/07/25 by D.Fujimoto removed rawCommands.
 * Modified on 2007/03/13 by D.Fujimoto refactored constructor and setDebuggerType() and
 * 										removed setXXXDefaults() and VEC_XXX_DEFAULTS.
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/07/19 by D.Fujimoto added ICD Ver6 attributes
 */
package jp.co.epson.ide.eclipse.gnu33.debug;

import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;

/**
 * This class represents the gdb launch settings.
 * Settings in the descriptor file is stored here and this object is 
 * stored in Gnu33Descriptor.
 * @author D.Fujimoto
 *
 */
public class DebuggerSetting {

	/**
	 * Debugger type constants
	 * Never delete these and do not assume these in order.
	 * These are not combo box indices !!
	 */
	public static final int TYPE_ICD2 	= 2;	
	public static final int TYPE_ICD3 	= 3;		// this is actually ICD3 and ICD4
	public static final int TYPE_MON 	= 1;
	public static final int TYPE_SIM 	= 0;
	public static final int TYPE_ICD6 	= 6;
	public static final int TYPE_ICDMINI = 100;
	public static final int TYPE_DEFAULT = TYPE_ICD3;

	// These UI Strings can be modified any time
	public static final String TYPE_ICD2_STR = "ICD Ver2";
	public static final String TYPE_ICD3_STR = "ICD Ver3/Ver4";
	public static final String TYPE_ICD6_STR = "ICD Ver6";
	public static final String TYPE_MON_STR = "MON";
	public static final String TYPE_SIM_STR = "Simulator";
	
	// These Project descriptor Strings need caution to be modified
	public static final String TYPE_ICD2_DESCSTR = "ICD2";
	public static final String TYPE_ICD3_DESCSTR = "ICD3";
	public static final String TYPE_ICD6_DESCSTR = "ICD6";
	public static final String TYPE_MON_DESCSTR = "MON";
	public static final String TYPE_SIM_DESCSTR = "SIM";
	
	/**
	 * Connect port constants 
	 */
	public static final int PORT_NONE = 0;
	public static final int PORT_USB = 1;
	public static final int PORT_COM = 2;
	public static final String PORT_NONE_STR = "NONE";
	public static final String PORT_USB_STR = "USB";
	public static final String PORT_COM_STR = "COM";
	public static final int COM_MAX = 8;
	
	/**
	 * Parallel port constants
	 */
	public static final int LPT_NONE = 0;
	public static final int LPT_1 = 1;
	public static final int LPT_2 = 2;
	public static final String LPT_NONE_STR = "NONE";
	public static final String LPT_1_STR = "LPT 1";
	public static final String LPT_2_STR = "LPT 2";
	
	/**
	 * Port wait constants
	 */
	public static final String WAIT_NONE_STR = "NONE";
	public static final int WAIT_MAX = 20;
	
	
	/**
	 * Debug RAM address default
	 */
	public static final long RAM_ADDR_DEFAULT = 0x71000;
	
	/**
	 * Baud rate defaults
	 */
	public static final int[] BAUD_RATES = {38400, 115200};
	public static final String BAUD_NONE_STR = "NONE";
	
	public static final String ERROR_MSG_DEFAULT = ResourceProvider.getResourceString("debuggerSetting.error");
	
	
	/**
	 * The settings name.
	 * Also used for the output file name.
	 */
	private String name = "default";
	
	/**
	 * Debugger type. ICDx | MON | SIM
	 */
	private int debuggerType;
		   
	/**
	 * The vector address
	 */
	private long vectorAddress;
	
	/**
	 * Flag whether to set the vector address
	 */
	private boolean useVectorAddress = true;
	
	/**
	 * Connect port. NONE(simulator) | USB(ICD VerX) | COM(ICD VerX/MON)
	 */
	private int connectPort;
	
	/**
	 * COM port number
	 */
	private int comPortNo;
		
	/**
	 * Parallel port number
	 */
	private int parallelPort;
   
	/**
	 * COM port access waits.
	 */
	private int portWait;
  
	/**
	 * Baud rate.
	 */
	private int baudRate;
	
	/**
	 * Debug RAM address(only for PE)
	 */
	private long debugRamAddr;
	
	
	/**
	 * Constructor.
	 */
	public DebuggerSetting() {
		super();
		setComPortNo(0);
		setParallelPort(LPT_NONE);
		setPortWait(0);
		setBaudRate(BAUD_RATES[1]);
		setDebugRamAddr(RAM_ADDR_DEFAULT);
	}
	
	/**
	 * Construcs a default setting given the CPU type and debugger type 
	 * @param debuggerType ICDx | MON | Simulator
	 * @since 1.4.0
	 */
	public DebuggerSetting(int debuggerType) {
		this();
		setDebuggerType(debuggerType);
	}
	
//	/**
//	 * Constructor.
//	 * Exclusively for creation from project wizard.
//	 * @param cpuType			STD or PE or ADV
//	 * @param debuggerType		ICD2 | ICD3 | MON | Simulator
//	 * @param setAddrFromStore	true to set vector address from the PreferenceStore
//	 * @since 1.3.0
//	 * @deprecated 1.4.0
//	 */
//	public DebuggerSetting(int cpuType, int debuggerType, boolean setAddrFromStore) {
//		this(cpuType, debuggerType);
//		
//		if (setAddrFromStore) {
//			// set the vector address value from the project wizard
//			IPreferenceStore store = Gnu33CorePlugin.getDefault().getPreferenceStore();
//			setVectorAddress(store.getLong("wizard.vecAddr"));
//		}
//	}

	
//	/**
//	 * Sets a default setting for Standard macro CPU type 
//	 * @deprecated
//	 */
//	public void setStdDefaults() {
//		setVectorAddress(VEC_STD_DEFAULT);
//
//		switch (debuggerType) {
//			case TYPE_ICD2 :
//				setConnectPort(PORT_COM);
//				setComPortNo(1);
//				break;
//			case TYPE_ICD3 :
//				setConnectPort(PORT_USB);
//				break;
//			case TYPE_MON :
//				setConnectPort(PORT_COM);
//				setComPortNo(1);
//				break;
//			case TYPE_SIM :
//				setConnectPort(PORT_NONE);
//				break;
//	 
//		}
//	}
//	
//	/**
//	 * Sets a default setting for PE macro CPU type
//	 * @deprecated
//	 */
//	public void setPeDefaults() {
//		setVectorAddress(VEC_PE_DEFAULT);
//		setDebugRamAddr(RAM_ADDR_DEFAULT);
//
//		switch (debuggerType) {
///* >>>>> DELETED by D.Fujimoto 2005/08/19 icd2 unsupported */ 
///*
//			case TYPE_ICD2 :
//				setConnectPort(PORT_COM);
//				setComPortNo(1);
//				setParallelPort(LPT_NONE);
//				setPortWait(0); 	   		
//				break;
//*/		  		
///* <<<<< DELETED by D.Fujimoto 2005/08/19 icd2 unsupported */
//			case TYPE_ICD3 :
//				setConnectPort(PORT_USB);
//				break;
//			case TYPE_SIM :
//				setConnectPort(PORT_NONE);
//				break;
//		}
//		
//	}
//	
//	/**
//	 * Sets a default setting for Advanced macro CPU type
//	 * @deprecated
//	 */
//	public void setAdvDefaults() {
//		setVectorAddress(VEC_ADV_DEFAULT);
//
//		
//		switch (debuggerType) {
//			case TYPE_ICD3 :
//				setConnectPort(PORT_USB);
//				break;
//
//			case TYPE_SIM :
//				setConnectPort(PORT_NONE);
//				break;
//		}
//	}
	
	/**
	 * Parses debugger type code to String.
	 * Used to write descriptor file.
	 * @param debuggerType		the code
	 * @return 	Debugger type as String
	 */
	public static String parseDebuggerTypeString(int debuggerType) {
		String ret = null;
		switch (debuggerType) {
			case TYPE_ICD2:
				ret = TYPE_ICD2_DESCSTR;
				break;
			case TYPE_ICD3:
				ret = TYPE_ICD3_DESCSTR;
				break;
			case TYPE_ICD6:
				ret = TYPE_ICD6_DESCSTR;
				break;
			case TYPE_MON:
				ret = TYPE_MON_DESCSTR;
				break;
			case TYPE_SIM:
				ret = TYPE_SIM_DESCSTR;
				break;
		}
		return ret;
	}
	
	/**
	 * Parses debugger type String to code.
	 * Used to read descriptor file.
	 * @param debuggerType		the String
	 * @return		Debugger type as code
	 */
	public static int parseDebuggerTypeInt(String debuggerType) {
		int ret = 0;
		if (debuggerType.equals(TYPE_ICD2_DESCSTR)) {
			ret = TYPE_ICD2;
		} else if (debuggerType.equals(TYPE_ICD3_DESCSTR)) {
			ret = TYPE_ICD3;
		} else if (debuggerType.equals(TYPE_ICD6_DESCSTR)) {
			ret = TYPE_ICD6;
		} else if (debuggerType.equals(TYPE_MON_DESCSTR)) {
			ret = TYPE_MON;
		} else if (debuggerType.equals(TYPE_SIM_DESCSTR)) {
			ret = TYPE_SIM;
		}
		return ret;
	}
	
	/**
	 * Parses connect port code to String.
	 * Used to write descriptor file.
	 * @param connectPort	the code
	 * @return 	Connect port as String
	 */
	public static String parseConnectPortString(int connectPort) {
		String ret = null;
		switch (connectPort) {
			case PORT_NONE:
				ret = PORT_NONE_STR;
				break;
			case PORT_USB:
				ret = PORT_USB_STR;
				break;
			case PORT_COM:
				ret = PORT_COM_STR;
				break;
		}
		return ret;
	}
	
	/**
	 * Parsses connect port String to code.
	 * Used to read descriptor file.
	 * @param connectPort	the String
	 * @return 	Connect port as code
	 */
	public static int parseConnectPortInt(String connectPort) {
		int ret = 0;
		if (connectPort.equals(PORT_NONE_STR)) {
			ret = PORT_NONE;
		} else if (connectPort.equals(PORT_USB_STR)) {
			ret = PORT_USB;
		} else if (connectPort.equals(PORT_COM_STR)) {
			ret = PORT_COM;
		}
		return ret;
	}
	
	/**
	 * Parses parallel port code to String.
	 * Used to write descriptor file.
	 * @param parallelPort		the code
	 * @return 	Parallel port as String
	 */
	public static String parseParallelPortString(int parallelPort) {
		String ret = null;
		switch (parallelPort) {
			case LPT_NONE :
				ret = "NONE";
				break;
			case LPT_1 :
				ret = "LPT1";
				break;
			case LPT_2 :
				ret = "LPT2";
				break;
		}
		return ret;
	}
	
	/**
	 * Parses parallel port String to code.
	 * Used to read descriptor file.
	 * @param parallelPort 		the String
	 * @return 	Parallel port as code
	 */
	public static int parseParallelPortInt(String parallelPort) {
		int ret = 0;
		if (parallelPort.equals("NONE")) {
			ret = LPT_NONE;
		} else if (parallelPort.equals("LPT1")) {
			ret = LPT_1;
		} else if (parallelPort.equals("LPT2")) {
			ret = LPT_2;
		}
		return ret;
	}
	
	
	// setters and getters
	public int getComPortNo() {
		return comPortNo;
	}
	public void setComPortNo(int comPortNo) {
		this.comPortNo = comPortNo;
	}
	public int getConnectPort() {
		return connectPort;
	}
	public void setConnectPort(int connectPort) {
		this.connectPort = connectPort;
	}
	public int getDebuggerType() {
		return debuggerType;
	}
	/**
	 * The debugger type will determine the port
	 * @param debuggerType
	 */
	public void setDebuggerType(int debuggerType) {
		this.debuggerType = debuggerType;
		
		switch (debuggerType) {
			case TYPE_ICD2 :
				setConnectPort(PORT_COM);
				setComPortNo(1);
				break;
			case TYPE_ICD3 :
				setConnectPort(PORT_USB);
				break;
			case TYPE_ICD6 :
				setConnectPort(PORT_USB);
				break;
			case TYPE_MON :
				setConnectPort(PORT_COM);
				setComPortNo(1);
				break;
			case TYPE_SIM :
				setConnectPort(PORT_NONE);
				break;
	 
		}
	}
	public int getParallelPort() {
		return parallelPort;
	}
	public void setParallelPort(int parallelPort) {
		this.parallelPort = parallelPort;
	}
	public int getPortWait() {
		return portWait;
	}
	public void setPortWait(int portWait) {
		this.portWait = portWait;
	}
	public long getVectorAddress() {
		return vectorAddress;
	}
	public void setVectorAddress(long vectorAddress) {
		this.vectorAddress = vectorAddress;
	}

	public int getBaudRate() {
		return baudRate;
	}
	public void setBaudRate(int baudRate) {
		this.baudRate = baudRate;
	}
	public long getDebugRamAddr() {
		return debugRamAddr;
	}
	public void setDebugRamAddr(long debugRamAddr) {
		this.debugRamAddr = debugRamAddr;
	}
	public boolean isUseVectorAddress() {
		return useVectorAddress;
	}
	public void setUseVectorAddress(boolean useVectorAddress) {
		this.useVectorAddress = useVectorAddress;
	}  
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Sets the setting name.
	 * @param name
	 * @since 1.2.0
	 */
	public void setDefaultSettingName(String name) {
		this.name = name + ResourceProvider.getResourceString("common.suffix");
	}


}
