/*
 * Created on 2004/09/07
 * Modified on 2005/06/16 by D.Fujimoto modified createDefault() to create SectionDatas
 * 											with autoUpdate flag Constructor. 
 * Modified on 2005/06/21 by D.Fujioto added methods updateAllObjList() and subtractOnjectList()
 * Modified on 2005/06/27 by D.Fujimoto added method checkDuplicateObjFiles() and related fields.		
 * 										modified methods addSection(), editSection() to use MessageFormat.	
 * Modified on 2005/07/12 by D.Fujimoto added method removeSection() with String arg.		
 * Modified on 2005/08/05 by D.Fujimoto modified constructor removed other DataPointer object creations. 
 * Modified on 2005/10/11 by D.Fujimoto added setDefaultSettingName() and modified Constructor(String).
 * Modified on 2005/11/09 by D.Fujimoto modified createDefault() quit setting selection flag to data pointer.
 * Modified on 2005/11/17 by D.Fujimoto deleted SectionComparator and changed the Comparator used in 
 * 											addSection().
 * Modified on 2005/11/18 by D.Fujimoto overloaded createDefault() for
 * 											initial vector section settings.
 * Modified on 2006/03/22 by D.Fujimoto modified createDefault() setting VMA done in project wizard. 
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/03/16 by D.Fujimoto createDefault() moved to CpuSettingsManager.
 * Modified on 2007/06/06 by D.Fujimoto lmaOverlaps() to prevent VMA and LMA overlapping
 */
package jp.co.epson.ide.eclipse.gnu33.linkerscript;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import jp.co.epson.ide.eclipse.gnu33.Gnu33CorePlugin;
import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.others.Logger;

/**
 * This class represents a LinkerScript Setting.
 * A LinkerScriptSetting has a Map of DataPointers and 
 * a list of SectionDatas.
 * Settings in the descriptor file is stored here and this object is 
 * stored in the descriptor file.
 * 
 * This class is the Mediator for adding/editing/removing SectionData objects.
 * 
 * @author D.Fujimoto
 *
 */
public class LinkerScriptSetting {

	// return codes for addSection(SectionData)
	public static final int OK = 1;
	public static final int NG = 0;
	public static final int ERR_DUPLICATE_SECTION = -1;
	public static final int ERR_NO_VMA_SECTION = -2;
	public static final int ERR_NO_LMA_SECTION = -3;
	public static final int ERR_LMA_OVERLAP = -4;
	public static final int ERR_REFERS_CYCLE = -5;
	public static final int ERR_REFERS_POST = -6;
	public static final int ERR_DUPLICATE_OBJ = -7;
	

	public static final String MSG_SEC_DELETE = LinkerScriptSettingResources.getString("delete.section.failed");
	public static final String MSG_DUPLICATE_SECTION = LinkerScriptSettingResources.getString("section.already.exists");
	public static final String MSG_NO_VMA_SECTION = LinkerScriptSettingResources.getString("error.vma.ref");
	public static final String MSG_NO_LMA_SECTION = LinkerScriptSettingResources.getString("error.lma.ref");
	public static final String MSG_LMA_OVERLAP = LinkerScriptSettingResources.getString("error.lma.overlap");
	public static final String MSG_REFERS_CYCLE = LinkerScriptSettingResources.getString("error.cycle.ref");
	public static final String MSG_REFERS_POST = LinkerScriptSettingResources.getString("error.invalid.ref");
	public static final String MSG_VMA_CYCLE = LinkerScriptSettingResources.getString("error.cycle.vma");
	public static final String MSG_LMA_CYCLE = LinkerScriptSettingResources.getString("error.cycle.lma");
	public static final String MSG_OBJ_DUPLICATED = LinkerScriptSettingResources.getString("error.obj.duplicated");
	
	/**
	 * The name of this setting.
	 * Also used as the file name for the linker script file.
	 */
	private String settingName = "default"; //$NON-NLS-1$
	
	/**
	 * List of DataPointers.
	 * Consists of 5 elements representing the data pointer's address.
	 * The order is D/G/Z/T/S
	 */
	private Map dataPointers;
	
	/**
	 * List of SectionData.
	 * (Never use List methods to add to/delete from this list unless it is an 
	 * internal operation). 
	 */
	private List sections;
	
	// error message containers
	String errorMessage;
	String overlappedSection;
	String badVmaSection;
	String badLmaSection;

	
	/**
	 * Creates a LinkerScriptSetting with no name. 
	 */
	public LinkerScriptSetting() {
		super();

		// create a data pointer Map
		dataPointers = new HashMap(5);
		dataPointers.put(DataPointer.TYPE_D_STR, new DataPointer(DataPointer.TYPE_D));
		
		// create an emply section List
		sections = new ArrayList();
		
	}
	
	/**
	 * Creates a LinkerScriptSetting with a name.
	 * @param settingName
	 */
	public LinkerScriptSetting(String settingName) {
		this();
		setDefaultSettingName(settingName);
	}
  
	
	/**
	 * Finds a SectionData from the list by name.
	 * @param sectionName the desired SectionDatas name
	 * @return the SectionData with the name, or null if not found
	 */
	public SectionData getSection(String sectionName) {
		for (int i = 0; i < sections.size(); i++) {
			SectionData aSection = (SectionData) sections.get(i);
			if (aSection.getSectionName().equals(sectionName)) {
				return aSection;
			}
		}
		return null;
	}
	
	/**
	 * Finds a SectionData from the list by name and returns its index in the list.
	 * @param sectionName the desired SectionDatas name
	 * @return the index, or -1 if not found
	 */
	public int getSectionIndex(String sectionName) {
		for (int i = 0; i < sections.size(); i++) {
			SectionData aSection = (SectionData) sections.get(i);
			if (aSection.getSectionName().equals(sectionName)) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Returns the section names from the sections contained in this setting.
	 * @return the section names
	 */
	public String[] getSectionNames() {
		String[] ret = new String[sections.size()];
		for (int i = 0; i < sections.size(); i++) {
			SectionData section = (SectionData) sections.get(i);
			ret[i] = section.getSectionName();
		}
		return ret;
	}
	
	/**
	 * Returns the vma end address symbol of the given section name.
	 * @param sectionName  
	 * @return the end address symbol as in "__END_text" for ".text" 
	 * 		or null if the section does not exist
	 */
	public String getEndSymbol(String sectionName) {
		
		SectionData aSection = getSection(sectionName);
		if (aSection != null) {
			return aSection.createLabels()[1];
		}
		return null;
		
	}
	
	/**
	 * Returns the lma address symbol of the given section name.
	 * @param sectionName 
	 * @return the lma address symbol as int "__START_text_lma" for ".text"
	 * 		or null if the section does not have an lma.
	 */
	public String getLmaSymbol(String sectionName) {
		
		SectionData aSection = getSection(sectionName);
		if (aSection != null && !aSection.isLMAequalsVMA()) {
			return aSection.createLabels()[2];
		}
		return null;
	}
	
	// getters and setters
	public Map getDataPointers() {
		return dataPointers;
	}
	public void setDataPointers(Map dataPointers) {
		this.dataPointers = dataPointers;
	}
	public List getSections() {
		return sections;
	}
	public void setSections(List sections) {
		this.sections = sections;
	}
	public String getSettingName() {
		return settingName;
	}
	public void setSettingName(String settingName) {
		this.settingName = settingName;
	}
	public String getErrorMessage() {
		return errorMessage;
	}
	
	/**
	 * Sets the default setting name.
	 * @param name
	 * @since 1.2.0
	 */
	public void setDefaultSettingName(String name) {
		setSettingName(name + ResourceProvider.getResourceString("common.suffix"));
	}
   
	/**
	 * Adds a section to the List of sections.
	 * Note that there are several conditions to add a section.
	 * @param newSection the SectionData to add
	 * @return one of the error codes defined in LinkerScriptSetting
	 */
	public int addSection(SectionData newSection) {

		String newSectionName = newSection.getSectionName();
		
		// ZNV̏d`FbN
		if (sectionNameExists(newSection) != -1) {
			errorMessage = MessageFormat.format(MSG_DUPLICATE_SECTION, new Object[] {newSectionName});
			return ERR_DUPLICATE_SECTION; // duplicate section name
		} 
		
		// QƂVMȂ݃`FbN
		if (!vmaSectionNameExists(newSection)) {
			errorMessage = MessageFormat.format(MSG_NO_VMA_SECTION, new Object[] {newSection.getVmaSectionName()});
			return ERR_NO_VMA_SECTION;
		}
		
		// QƂLMȂ݃`FbN
		if (!lmaSectionNameExists(newSection)) {
			errorMessage = MessageFormat.format(MSG_NO_LMA_SECTION, new Object[] {newSection.getLmaSectionName()});
			return ERR_NO_LMA_SECTION;
		}
		
		// LMȀd`FbN
		if (lmaOverlaps(newSection)) {
			errorMessage = MessageFormat.format(MSG_LMA_OVERLAP, new Object[] {newSectionName, overlappedSection}); 
			return ERR_LMA_OVERLAP;	// lma overlaps with another section
		}

		
		sections.add(newSection);
		Collections.sort(sections, new VmaSectionComparator(sections)); 	 

		// QƂ̃`FbN
		if (refersPost(newSection)) {
			errorMessage = MessageFormat.format(MSG_REFERS_POST, new Object[] {newSection.getSectionName()});
			sections.remove(newSection);
			return ERR_REFERS_POST;
		}
		
		return OK;	// OK
	
	}
	
	
	/**
	 * Checks if the newSection's section name exists in the list.
	 * The section names are compared without the leading ".".
	 * Meaning, ".bss" and "bss" are considered equal.
	 * @param newSection the section to check
	 * @return the list's index in which this section name exists, -1 if not found.
	 * @see LinkerScriptSetting#addSection(SectionData)
	 */
	private int sectionNameExists(SectionData newSection) {
		int foundIndex = -1;
		
		String newSectionName = newSection.getSectionName();
		if (newSectionName.startsWith(".")) {
			newSectionName = newSectionName.substring(1);
		}
		for (int i = 0; i < sections.size(); i++) {
			String sectionName = ((SectionData) sections.get(i)).getSectionName();
			if (sectionName.startsWith(".")) { //$NON-NLS-1$
				sectionName = sectionName.substring(1);
			}
			if (newSectionName.equals(sectionName)) {
				foundIndex = i;
				break;
			}
		}
		return foundIndex;
	}
	
	/**
	 * Checks if the newSection's vmaSectionName exists in the list.
	 * Performs a check only if the newSection's vmaIsAddress flag is false.
	 * @param newSection the section to check
	 * @return true if the section name exists, otherwise false.
	 * @see LinkerScriptSetting#addSection(SectionData)
	 */ 
	private boolean vmaSectionNameExists(SectionData newSection) {
		if (!newSection.isVmaIsAddress()) {
			boolean found = false;
			String[] secNames = getSectionNames();
			for (int i = 0; i < secNames.length; i++) {
				if(newSection.getVmaSectionName().equals(secNames[i])) {
					found = true;
					break;
				}
			}
			return found;	// false when referenced vma section does not exist

		} else {
			// check omitted
			return true;   
		}
	}
	
	/**
	 * Checks if the newSection's lmaSectionName exists in the list.
	 * Performs a check only if the newSection's LMAequalsVMA flag and 
	 * lmaIsAddress flag are both false.
	 * @param newSection the section to check
	 * @return true if the section name exists, otherwise false.
	 * @see LinkerScriptSetting#addSection(SectionData)
	 */
	private boolean lmaSectionNameExists(SectionData newSection) {
		if (!newSection.isLMAequalsVMA() && !newSection.isLmaIsAddress()) {
			boolean found = false;
			String[] secNames = getSectionNames();
			for (int i = 0; i < secNames.length; i++) {
				if (newSection.getLmaSectionName().equals(secNames[i])) {
					found = true;
					break;
				}
			}
			return found;	// false when referenced lma section does not exist 
		} else {
			// check omitted
			return true;
		}
	}
	
	/**
	 * Checks if the newSection's lma overlapps with another section's lma in the list.
	 * When this method returns true, the new section cannot be added to the list.
	 * <p>For sections that are isLmaEqualsVma, either vma or vmaSectionName is considered as the 
	 * lma value.
	 * <p>If it overlaps, the overlapped section name is stored 
	 * in LinkerScriptSetting#overlappedSection.
	 * 
	 * Also, if the newSection and another section are both isLmaEqualsVma = true, 
	 * the check is omitted.
	 * @param newSection the section to check
	 * @return true if the newSection lma overlapps, otherwise false.
	 * @see LinkerScriptSetting#addSection(SectionData)
	 * @see LinkerScriptSetting#overlappedSection
	 */
	private boolean lmaOverlaps(SectionData newSection) {
		// initialize the error message
		overlappedSection = ""; //$NON-NLS-1$

		// used when the new section's lma is an address
		long newLmaAddress = -1;
		// used when the new section's lma is a section name
		String newLmaSection = null;
		
		// get the new section's lma as an address or the referenced section name
		if (newSection.isVmaIsAddress()) {
			if (newSection.isLMAequalsVMA()) {
				newLmaAddress = newSection.getVma();
			} else {
				if (newSection.isLmaIsAddress()) {
					newLmaAddress = newSection.getLma();
				} else {
					newLmaSection = newSection.getLmaSectionName();
				}
			}
		} else {
			if (newSection.isLMAequalsVMA()) {
				newLmaSection = newSection.getVmaSectionName();
			} else {
				if (newSection.isLmaIsAddress()) {
					newLmaAddress = newSection.getLma();
				} else {
					newLmaSection = newSection.getLmaSectionName();
				}
			}
		}
		
		// check if it overlaps with other section's lma
		if (newLmaAddress != -1) {
			for (int i = 0; i < sections.size(); i++) {
				SectionData otherSection = (SectionData) sections.get(i);
				if (otherSection.isLMAequalsVMA()) {
					if (otherSection.isVmaIsAddress() && !newSection.isLMAequalsVMA()) {
						// check it with the vma address
						if (newLmaAddress == otherSection.getVma()) {
							overlappedSection = otherSection.getSectionName();
							return true;
						}
					} 
				} else {
					if (otherSection.isLmaIsAddress()) {
						// check it with the lma address
						if (newLmaAddress == otherSection.getLma()) {
							overlappedSection = otherSection.getSectionName();
							return true;
						}
					}
					
				}
			}
		} else if (newLmaSection != null) {
			for (int i = 0; i < sections.size(); i++) {
				SectionData otherSection = (SectionData) sections.get(i);
				/* >>>>> ADDED by D.Fujimoto 2007/06/06 added vma-ref and lma-ref check */
				// this check is deleted-- 2004.10.26
				if (otherSection.isLMAequalsVMA()) {
					if (!otherSection.isVmaIsAddress() && !newSection.isLMAequalsVMA()) {
						// check it with the vma reference
						if (newLmaSection.equals(otherSection.getVmaSectionName())) {
							overlappedSection = otherSection.getSectionName();
							return true;
						}
					}
/* <<<<< ADDED by D.Fujimoto 2007/06/06 added vma-ref and lma-ref check */
				} else 	if (!otherSection.isLMAequalsVMA()) {	 
					if (!otherSection.isLmaIsAddress() && !newSection.isLMAequalsVMA()) {
						// check it with the lma reference
						if (newLmaSection.equals(otherSection.getLmaSectionName())) {
							overlappedSection = otherSection.getSectionName();
							return true;
						}
/* >>>>> ADDED by D.Fujimoto 2007/06/06 added vma-ref and lma-ref check */
					} else if (!otherSection.isLmaIsAddress() && newSection.isLMAequalsVMA()) {
						if (newLmaSection.equals(otherSection.getLmaSectionName())) {
							overlappedSection = otherSection.getSectionName();
							return true;
						}
/* <<<<< ADDED by D.Fujimoto 2007/06/06 added vma-ref and lma-ref check */
					}
				}
			}
		
		}
		
		// the new section's lma does not overlap with any section in the list
		return false;
		
	}

	/**
	 * Checks if object list of the given SectionData contains duplicate
	 * entries as specified in another section.<br>
	 * The duplicated file and section will be set in the errorMessage field.
	 * 
	 * The comparison will be performed if:
	 * <ul>the given section is auto update disabled</ul>
	 * <ul>the other section is auto update disabled</ul>
	 * <ul>the sections have the same attribute</ul>
	 * 
	 * This method will return immediately when the first duplication is
	 * found.
	 * 
	 * @param newSection
	 *			  the section currently being edited
	 * @param originalName	The section that we don't want to compare. 
	 * 			The section's name might be changed if it is edited.
	 * @return true if the given section has duplicates
	 * @since 1.1.1
	 */
	public boolean checkDuplicateObjFiles(SectionData newSection, String originalName) {

		// initialize the error message
		errorMessage = null;

		// no checks if the section is auto update mode
		if (newSection.isAutoUpdate()) {
			return false;
		}

		String attribute = newSection.getAttribute();
		java.util.List objList = newSection.getObjectList();
		if (objList.isEmpty()) {
			return false;
		}

		for (int i = 0; i < sections.size(); i++) {
			SectionData otherSection = (SectionData) sections.get(i);

			// skip the currently editing section
			if (otherSection.getSectionName().equals(originalName)) {
				continue;
			}
			
			// checks if the other section is not in auto update mode
			// and the attribute is the same
			if (!otherSection.isAutoUpdate()
					&& attribute.equals(otherSection.getAttribute())) {
				java.util.List otherObjList = otherSection.getObjectList();

				// check for any duplicates in this section
				for (int j = 0; j < otherObjList.size(); j++) {
					String objName = (String) otherObjList.get(j);

					if (objList.contains(objName)) {
						// duplicate found!
						// set the error message
						errorMessage = MessageFormat.format(MSG_OBJ_DUPLICATED,
								new Object[] { objName, otherSection.getSectionName() });
						return true;
					}
				}

			}
		}

		return false;
	}
	
	/**
	 * Checks if the newSection points its vmaSectionName after where it is in
	 * the section list.
	 * <p>
	 * For example, if the list contains sections .bss, .text in this order,
	 * .bss cannot state its vmaSectionName as .text, and this method will
	 * return true.
	 * <p>
	 * Checks will not be performed when newSection's LMAequalsVMA flag and
	 * lmaIsAddress flag are both false.
	 * 
	 * This method should be called after the newSection is added to the list
	 * and is sorted.
	 * 
	 * @param newSection
	 *			  the section to check
	 * @return true when this section refers posterior, otherwise false.
	 */
	private boolean refersPost(SectionData newSection) {
		if (!newSection.isLMAequalsVMA() && !newSection.isLmaIsAddress()) {
			// omit check
			return false;
		}
		
		// get the index of the list 
		int thisIndex = getSectionIndex(newSection.getSectionName());
		
		// if it refers vma
		if (!newSection.isVmaIsAddress()) {
			int thatIndex = getSectionIndex(newSection.getVmaSectionName());
			if (thisIndex < thatIndex) {
				// refers vma
				return true;
			}
		}
		
		return false;
	}
	
	
	/**
	 * Removes and adds the edited section to the list.
	 * Restore the list as it was if an error occurs while adding the section.
	 * @param index the index of this section 
	 * @param editSection the edited section
	 * @return the return value of addSection(SectionData)
	 */
	public int editSection(int index, SectionData editSection) {

		// clone the whole section list which will be restored when an error occurs
		List cache = new ArrayList();
		for (int i = 0; i < sections.size(); i++) {
			SectionData aSection = (SectionData) sections.get(i);
			cache.add(new SectionData(aSection));
		}
		
		// this is the section list before it gets edited
		// NOTE:this is different from "cache", because we are going to use
		// List.contains(SectionData) inside checkLeadingVma() and checkLeadingLma()
		List oldSectionList = new ArrayList(sections);
		
		SectionData oldSection = (SectionData) sections.get(index);
		// see if the edit results into a vma reference cycle
		if (vmaHasRefCycles(oldSection, editSection, oldSectionList)) {
			// if it does...restore the original section list
			errorMessage = MessageFormat.format(MSG_VMA_CYCLE, new Object[] {editSection.getSectionName(), badVmaSection}); 
			sections = cache;
			return ERR_REFERS_CYCLE;
		}
		
		// see if the edit results into an lma reference cycle
		if (lmaHasRefCycles(oldSection, editSection, oldSectionList)) {
			// if it does...restore the original section list
			errorMessage = MessageFormat.format(MSG_LMA_CYCLE, new Object[] {editSection.getSectionName(), badLmaSection}); 
			sections = cache;
			return ERR_REFERS_CYCLE;
		}
		
	  
		// remove the now-editing section
		SectionData originalSection = (SectionData) sections.remove(index);

		// change the vma/lma references in case the section name gets changed
		for (int i = 0; i < sections.size(); i++) {
			SectionData aSection = (SectionData) sections.get(i);
			if (!aSection.isVmaIsAddress() && aSection.getVmaSectionName().equals(originalSection.getSectionName())) {
				aSection.setVmaSectionName(editSection.getSectionName());
			}
			if (!aSection.isLMAequalsVMA() && !aSection.isLmaIsAddress() &&
					aSection.getLmaSectionName().equals(originalSection.getSectionName())) {
				aSection.setLmaSectionName(editSection.getSectionName());
			}
		}
		
		// add the edited section and sort the list
		int result = addSection(editSection);

		// restore the list if a problem occurs
		if (result != OK) {
			if (Gnu33CorePlugin.isDebug) {
				System.out.println("EDIT ABORTED");
			}
			sections = cache;
		}

		return result;
		
	}
	
	/**
	 * Checks if the edited section has VMA section cycles.
	 * Checks are only performed when the edited section's VMA is a reference.
	 * @param oldSection	the section as it was before being edited
	 * @param newSection	the section as it is after being edited
	 * @param oldSectionList	the original section list before being edited
	 * @return true if the section may have cycles
	 */
	private boolean vmaHasRefCycles(SectionData oldSection, SectionData newSection, List oldSectionList) {
		// get the section name before it was edited
		String oldSectionName = oldSection.getSectionName();

		// section names which will cause a cycle reference will be 
		// stored in this list
		List badRefList = new ArrayList();
		
		// omit check
		if (newSection.isVmaIsAddress()) {
			return false;
		}
		
		// find sections whose vma reference lead to the edited section
		for (int i = 0; i < oldSectionList.size(); i++) {
			SectionData section = (SectionData) oldSectionList.get(i);
			if (!oldSectionName.equals(section.getSectionName())) {// don't compare with itself
				if (!section.isVmaIsAddress()) {
					// check if the section's vma lead to the edited section
					if (checkLeadingVma(section, oldSectionName)) {
						badRefList.add(section.getSectionName());
					}
				}
			}
		}
		
		// see if the edited section references a bad section
		String newVmaSectionName = newSection.getVmaSectionName();
		for (int i = 0; i < badRefList.size(); i++) {
			String badSectionName = (String) badRefList.get(i); 
			if (newVmaSectionName.equals(badSectionName)) {
				badVmaSection = badSectionName;
				return true;
			}
		}
		
		return false;
   
	}

	/**
	 * Checks if the section's vma leads to the section given in the second argument.
	 * This method will be called recursively.
	 * 
	 * If the section's vma is a real address, this method 
	 * returns false.
	 * @param section the section to be checked
	 * @param vmaReference	the vma String this section shouldn't have
	 * @return	true if the given section leads to vmaReference
	 */
	private boolean checkLeadingVma(SectionData section, String vmaReference) {
		
		// the section must be in the list
		if (!sections.contains(section)) {
			return false;
		}
		
		if (section.isVmaIsAddress()) {
			return false;
		} else {
			String vmaSectionName = section.getVmaSectionName();
			if (vmaReference != null && vmaReference.equals(vmaSectionName)) {
				return true;
			}

			// if it references another section, find recursively
			SectionData referencedSection = getSection(vmaSectionName);
			if (referencedSection != null) {
				return checkLeadingVma(referencedSection, vmaReference);
			}
		}
		
		// shouldn't come this far
		return false;
		
	}  
	
	/**
	 * Checks if the edited section has LMA section cycles.
	 * Checks are only performed when the edited section's LMA is a reference.
	 * @param oldSection	the section as it was before being edited
	 * @param newSection	the section as it is after being edited
	 * @param oldSectionList	the original section list before being edited
	 * @return true if the section may have cycles
	 */
	private boolean lmaHasRefCycles(SectionData oldSection, SectionData newSection, List oldSectionList) {
		// get the section name before it was edited
		String oldSectionName = oldSection.getSectionName();

		// section names which will cause a cycle reference will be 
		// stored in this list
		List badRefList = new ArrayList();
		
		// omit check
		if (newSection.isLMAequalsVMA()) {
			return false;
		} else if (newSection.isLmaIsAddress()) {
			return false;
		}
		
		// find sections whose lma reference lead to the edited section
		for (int i = 0; i < oldSectionList.size(); i++) {
			SectionData section = (SectionData) oldSectionList.get(i);
			if (!oldSectionName.equals(section.getSectionName())) {// don't compare with itself
				if (!section.isLMAequalsVMA() && !section.isLmaIsAddress()) {
					// check if the section's lma lead to the edited section
					if (checkLeadingLma(section, oldSectionName)) {
						badRefList.add(section.getSectionName());
					}
				}
			}
		}
		
		// see if the edited section references a bad section
		String newLmaSectionName = newSection.getLmaSectionName();
		for (int i = 0; i < badRefList.size(); i++) {
			String badSectionName = (String) badRefList.get(i); 
			if (newLmaSectionName.equals(badSectionName)) {
				badLmaSection = badSectionName;
				return true;
			}
		}
		
		return false;
   
	}

	/**
	 * Checks if the section's lma leads to the section given in the second argument.
	 * This method will be called recursively.
	 * 
	 * If the section is LMA=VMA, or the section's lma is a real address, this method 
	 * returns false.
	 * @param section the section to be checked
	 * @param lmaReference	the lma String this section shouldn't have
	 * @return	true if the given section leads to lmaReference
	 */
	private boolean checkLeadingLma(SectionData section, String lmaReference) {
		
		// the section must be in the list
		if (!sections.contains(section)) {
			return false;
		}
		
		if (section.isLMAequalsVMA() || (!section.isLMAequalsVMA() && section.isLmaIsAddress())) {
			return false;
		} else {
			String lmaSectionName = section.getLmaSectionName();
			if (lmaReference != null && lmaReference.equals(lmaSectionName)) {
				return true;
			}

			// if it references another section, find recursively
			SectionData referencedSection = getSection(lmaSectionName);
			if (referencedSection != null) {
				return checkLeadingVma(referencedSection, lmaReference);
			}
		}
		
		// shouldn't come this far
		return false;
		
	}  

	/**
	 * Removes the section from the List of sections. 
	 * @param delSection the section to delete
	 * @return OK or NG, either way the section will be deleted from the list
	 */
	public int removeSection(SectionData delSection) {

		// get the index in the list
		String delSectionName = delSection.getSectionName();
		int index = getSectionIndex(delSectionName);
		
		try {
			// remove it from the list
			sections.remove(index);
	
			// do some reference corrections
			for (int i = 0; i < sections.size(); i++) {
				SectionData aSection = (SectionData) sections.get(i);
				
				// automatically rewrite any section's vma which refer to the 
				// deleted section
				if (!aSection.isVmaIsAddress() && aSection.getVmaSectionName().equals(delSectionName)) {
					if (delSection.isVmaIsAddress()) {
						aSection.setVmaSectionName(""); //$NON-NLS-1$
						aSection.setVma(delSection.getVma());
					} else {
						aSection.setVma(0);
						aSection.setVmaSectionName(delSection.getVmaSectionName());
					}	 
				}
				
				// automatically rewrite any section's lma which refer to the 
				// deleted section
				if (!aSection.isLMAequalsVMA() && !aSection.isLmaIsAddress() && 
						aSection.getLmaSectionName().equals(delSectionName)) {
					if (delSection.isLMAequalsVMA()) {
						if (delSection.isVmaIsAddress()) {
							aSection.setLmaSectionName(""); //$NON-NLS-1$
							aSection.setLma(delSection.getVma());
						} else {
							aSection.setLma(0);
							aSection.setLmaSectionName(delSection.getVmaSectionName());
						}
					} else {
						if (delSection.isLmaIsAddress()) {
							aSection.setLmaSectionName(""); //$NON-NLS-1$
							aSection.setLma(delSection.getLma());
						} else {
							aSection.setLma(0);
							aSection.setLmaSectionName(delSection.getLmaSectionName());
						}
					}
				}
			}
			
		} catch (Exception e) {
			Logger.log(MSG_SEC_DELETE + delSectionName);
			return NG;
		}
		
		
		return OK;
	}
	
	/**
	 * A service method to remove a section by its name.
	 * If the section does not exist it will do nothing.
	 * 
	 * @param sectionName	the section's name to be deleted 
	 * @since 1.1.1
	 */
	public void removeSection(String sectionName) {
		
		// retrieve by name
		SectionData delSection = getSection(sectionName);
		
		if (delSection != null) {
			removeSection(delSection);
		}
	}
	
	/**
	 * Updates the object list for all sections in this setting which are marked as 
	 * auto update.
	 * @param objFiles		the object file list assumed from the members of 
	 * 		the project 
	 * @since 1.1.1
	 */
	public void updateAllObjList(List objFiles) {

		// get the sections from the linker setting
		List sectionList = getSections();
		for (int i = 0; i < sectionList.size(); i++) {
			SectionData sectionData = (SectionData) sectionList.get(i);

			// update the sections that are marked auto update
			if (sectionData.isAutoUpdate()) {
				// do not contain files that are contained 
				// in another section with the same attribute
				List subList = subtractObjectList(objFiles, sectionData, sectionData.getAttribute());
				sectionData.setObjectList(subList);
			}
		}
	}
	
	/**
	 * Returns an object list from the members of the project.
	 * This list will drop the object files that are contained in 
	 * another section with the same attribute.(like .bss or .text)
	 * The SectionData in the second argument will not be checked.	
	 * @param objList	the object file list assumed from the members of 
	 * 		the project 
	 * @param	skip 	the SectionData that should be skipped
	 * @param sectionAttribute		the section's attribute
	 * @since 1.1.1
	 */

	public List subtractObjectList(List objList, SectionData skip, String sectionAttribute) {

		// the following seemingly irrelevant line is not to be deleted
		List retList = new ArrayList(objList);
		
		// check every section
		for (int i = 0; i < sections.size(); i++) {
			SectionData section = (SectionData) sections.get(i);

			// skip the section with the same name
			if (section.getSectionName().equals(skip.getSectionName())) {
				continue;
			}
			
			// is it the same attribute?
			if (section.getAttribute().equals(sectionAttribute)) {
				List aList = section.getObjectList();
				for (int j = 0; j < aList.size(); j++) {
					String objName = (String) aList.get(j);
					// is it the same name?
					if (retList.contains(objName)) {
						// remove it
						retList.remove(objName);
					}
				}
			}
		}
		
		return retList;
		
	}
	
	
	void check() {
		for (Iterator iter = sections.iterator(); iter.hasNext();) {
			SectionData element = (SectionData) iter.next();
			
			System.out.println("checking " + element.getSectionName());
			boolean ret = false;
			
			ret = vmaSectionNameExists(element);
			System.out.println("vma:" + ret);
			
			ret = lmaSectionNameExists(element);
			System.out.println("lma:" + ret);
			
			ret = refersPost(element);
			System.out.println("post:" + !ret);
			System.out.println();
			
		}	
	}
   
}
