/*
 * 쐬: 2005/11/24
 *
 */
package jp.co.epson.ide.eclipse.gnu33.linkerscript;

import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Comparator for sorting the SectionDatas in LMA order.
 * If a section's vma or lma is a reference to another section,
 * the order depends on the nearest real vma of lma address
 * 
 * @author D.Fujimoto
 * @since 1.2.0
 *
 */
public class LmaSectionComparator implements Comparator {

	static final String SEC1 = "sec1"; //$NON-NLS-1$
	static final String SEC2 = "sec2"; //$NON-NLS-1$
	
	// counter for how many times getNearestLmaAddress() was invoked
	Map counterMap = new HashMap();
	
	// the List of SectionData to search recursively
	private List sections;
	
	// The type of a section
	// contains information for comparing one another
	private class AddrType {
	
		public static final int LMA_ADDR = 1;	// section has an LMA address	
		public static final int LMA_REF = 2;		// section has an LMA reference
		public static final int VMA_ADDR = 3;	// section has a VMA address
		public static final int VMA_REF = 4;		// section has a VMA reference
		
		long address;
		int kind;
		Integer refCount = new Integer(0);		// used when LMA_REF or VMA_REF
		
	}
	
	/**
	 * Constructor.
	 * Needs the List of SectionData to do recursive searches
	 * @param sections
	 */
	public LmaSectionComparator(List sections) {
		this.sections = sections;
	}
	
	
	/**
	 * Compares SectionData by their lma address or, the referenced section.
	 * @return -1 when o1 should come before o2 in the list, (o1=direct address)
	 * 		1 when o2 should come before o1 in the list (o1=ref/o2=direct address or ref),
	 * 		0 when o1 and o2 are considered equal(not likely to happen).
	 * 
	 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
	 */
	public int compare(Object o1, Object o2) {
		int result = -1;
		
		SectionData s1 = (SectionData) o1;
		SectionData s2 = (SectionData) o2;
		
		// reset counter
		counterMap.put(SEC1, new Integer(0));
		counterMap.put(SEC2, new Integer(0));
		
		// get the types for both
		AddrType type1 = getType(s1, SEC1);
		AddrType type2 = getType(s2, SEC2);
		
		
		if (type1.kind == AddrType.LMA_ADDR || type1.kind == AddrType.VMA_ADDR) {
			result = new Long(type1.address).compareTo(new Long(type2.address));
			if (result == 0) {
				// the real address comes first(irrelevant of what type2 is)
				result = -1;
			}
		} else {
			// kind is AddrType.LMA_REF or AddrType.VMA_REF
			result = new Long(type1.address).compareTo(new Long(type2.address));
			if (result == 0) {
				if (type2.kind == AddrType.LMA_REF || type2.kind == AddrType.VMA_REF) {
					// if both are references, compare by refCount
					result = type1.refCount.compareTo(type2.refCount);
				} else {
					// if type2 is an address, the real address comes first
					result = 1;
				}
			}
			
			
		}
		
		return result;
	}
	
	/**
	 * Returns the AddrType for this section.
	 * 
	 * @param sec	the SectionData in question
	 * @param key	the key to store in counterMap
	 * @return the AddrType expression of the given SectionData
	 */
	private AddrType getType(SectionData sec, String key) {
		
		AddrType type = new AddrType();
		
		if (sec.isLMAequalsVMA()) {
			if (sec.isVmaIsAddress()) {
				type.kind = AddrType.VMA_ADDR;
				type.address = sec.getVma();
			} else {
				// recursive search for the address
				type.kind = AddrType.VMA_REF;
				type.address = getNearestLmaAddress(sec, key);
				type.refCount = (Integer) counterMap.get(key);	
			}
			
		} else {
			if (sec.isLmaIsAddress()) {
				type.kind = AddrType.LMA_ADDR;
				type.address = sec.getLma();
			} else {
				// recursive search for the address
				type.kind = AddrType.LMA_REF;
				type.address = getNearestLmaAddress(sec, key);
				type.refCount = (Integer) counterMap.get(key);
			}
		
		}
		
		
		return type;
		
	}
	
	/**
	 * Returns the given section's nearest real lma address.
	 * If this section's lma is not a real address, this method 
	 * recursively searches until it finds the real address.
	 * <p>
	 * Counts up how many times this method was invoked and stores it in a Map by the 
	 * given key.
	 * @param section the SectionData in question
	 * @param key the key to the counterMap
	 * @return the address or -1 if it cannot find
	 */
	private long getNearestLmaAddress(SectionData section, String key) {

		// count up
		int count = ((Integer) counterMap.get(key)).intValue();
		count++;
		counterMap.put(key, new Integer(count));
		
		// the section must be in the list
		if (!sections.contains(section)) {
			return -1;
		}
		 
		if (section.isLMAequalsVMA()) {
			if (section.isVmaIsAddress()) {
				return section.getVma();
			} else {
				// get the ref VMA section
				SectionData aSection = getSectionByName(section.getVmaSectionName());
				
				// avoid endless
				if (aSection != null) {
					if (!section.getSectionName().equals(aSection.getVmaSectionName())) {
						return getNearestLmaAddress(aSection, key);
					}
				}
			}
			
		} else {
			if (section.isLmaIsAddress()) {
				return section.getLma();
			} else {
				// get the ref LMA section
				SectionData aSection = getSectionByName(section.getLmaSectionName());
				
				// avoid endless
				if (aSection != null) {
					if (!section.getSectionName().equals(aSection.getLmaSectionName())) {
						return getNearestLmaAddress(aSection, key);
					}
				}
			}
		
		}
		
		// shouldn't come this far
		return -1;

	}
	
	/**
	 * Finds a SectionData from the list by name.
	 * @param sectionName the desired SectionDatas name
	 * @return the SectionData with the name, or null if not found
	 */
	public SectionData getSectionByName(String sectionName) {
		for (int i = 0; i < sections.size(); i++) {
			SectionData aSection = (SectionData) sections.get(i);
			if (aSection.getSectionName().equals(sectionName)) {
				return aSection;
			}
		}
		return null;
	}
	
	
/*	  
	public static void main(String[] args) {
		
		boolean auto = true;
		
		// add sections for C++
		// bss section
		SectionData bss = new SectionData(SectionData.ATTR_BSS, auto);
		bss.setVma(0x00);
		bss.setVmaIsAddress(true);
		
		// comm section (auto update should be true)
		SectionData comm = new SectionData(SectionData.ATTR_COMM, true);
		comm.setVmaSectionName(bss.getSectionName());

	   
		// create text section
		SectionData text = new SectionData(SectionData.ATTR_TEXT, auto);
		text.setVma(0xC00000);
		text.setVmaIsAddress(true);
		
		// rodata section
		SectionData rodata = new SectionData(SectionData.ATTR_RODATA, auto);
		rodata.setVmaSectionName(text.getSectionName());
		rodata.setVmaIsAddress(false);
		
		// ctors section (auto update should be true)
		SectionData ctors = new SectionData(SectionData.ATTR_CTORS, true);
		ctors.setVmaSectionName(rodata.getSectionName());
		
		// dtors section (auto update should be true)
		SectionData dtors = new SectionData(SectionData.ATTR_DTORS, true);
		dtors.setVmaSectionName(ctors.getSectionName());	   

		// ctors section (auto update should be true)
		SectionData except = new SectionData(SectionData.ATTR_GCC_EXCEPTION, true);
		except.setVmaSectionName(dtors.getSectionName());
		
		// data section
		SectionData data = new SectionData(SectionData.ATTR_DATA, auto);
		data.setVmaSectionName(comm.getSectionName());
		data.setVmaIsAddress(false);
		data.setLMAequalsVMA(false);
		data.setLmaSectionName(except.getSectionName());
		data.setLmaIsAddress(false);
		
		List vmaSecs = new ArrayList();
		vmaSecs.add(bss);
		vmaSecs.add(comm);
		vmaSecs.add(text);
		vmaSecs.add(rodata);
		vmaSecs.add(ctors);
		vmaSecs.add(dtors);
		vmaSecs.add(except);
		vmaSecs.add(data);
		
		VmaSectionComparator vComp = new VmaSectionComparator(vmaSecs);
		Collections.sort(vmaSecs, vComp);
  	
		// do not affect the vma order List
		List lmaSecs = new ArrayList(vmaSecs);
		
		
		LmaSectionComparator comp = new LmaSectionComparator(lmaSecs);
		Collections.sort(lmaSecs, comp);
		
		System.out.println("LMA order");
		for (Iterator iter = lmaSecs.iterator(); iter.hasNext();) {
			SectionData element = (SectionData) iter.next();
			System.out.println(element.getSectionName());
		}
		
		System.out.println("VMA order");
		for (Iterator iter = vmaSecs.iterator(); iter.hasNext();) {
			SectionData element = (SectionData) iter.next();
			System.out.println(element.getSectionName());
		}
	
	} 
*/

}
