/*
 * Created on 2004/08/31
 * Modified on 2005/06/16 by D.Fujimoto added field autoUpdate and its
 * 											get/set operations.
 * Modified on 2005/07/08 by D.Fujimoto added new sections for C++
 * 										added method needsAlign().
 * Modified on 2005/08/01 by D.Fujimoto added method getAttributeOutputString().
 * Modified on 2005/08/08 by D.Fujimoto deleted int constants concerning data pointers
 * 										deleted values from ATTR_NAMES.
 * Modified on 2006/02/20 by D.Fujimoto changed method createLabels() added lma end label.
 * Modified on 2006/03/20 by D.Fujimoto added isDefaultSection().
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.linkerscript;

import java.util.ArrayList;
import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.file.AbstractFileGenerator;

/**
 * This class represents a SectionData for a Linker Script.
 * This class is contained in LinkerScriptSetting.
 * @author D.Fujimoto
 *
 */
public class SectionData {

	/**
	 * The section's name.
	 * This will be written to the .lds file.
	 */
	private String sectionName;
	
	/**
	 * Flag if vma contains a valid address.
	 * true when this section's address is specified in vma.
	 * false when this section's address is not specified.
	 * In that case, this section comes after the leading section in.
	 * LinkerScriptSetting#sections  
	 */
	private boolean vmaIsAddress;
	
	/**
	 * The section's virtual map address.
	 * This address is the main address to be shown.
	 * This is set when vmaIsAddress == true.
	 */
	private long vma;
  
	/**
	 * The section's vma reference.
	 * This is set when vmaIsAddress == false.
	 */
	private String vmaSectionName;
	
	/**
	 * Indicates whether LMA is specified.
	 * true when lma is specified, false when not specified.
	 * The default is true.
	 * No need to reference lmaIsAddress | lma | lmaSectionName 
	 * when this is set to true.
	 */
	private boolean LMAequalsVMA;

	/**
	 * Flag if lma contains a valid address.
	 * true when this section's address is specified in lma.
	 * false when this section's address is not specified.
	 * In that case, the referenced section will be contained in
	 * lmaSectionName.
	 */
	private boolean lmaIsAddress;
	
	/**
	 * The section's load map address.
	 * This is set when lmaIsAddress == true.
	 */
	private long lma;
	
	/**
	 * The section's lma reference.
	 * This is set when lmaIsAddress == false. 
	 */
	private String lmaSectionName;
	
	/**
	 * The section's label to be used in the user source code.
	 * Array0 is the start label and array1 is the end label.
	 * Array2 is the lma start label and array3 is the lma end label.
	 */
	private String[] sectionLabels = new String[4];
		
	/**
	 * The section's attribute.
	 * This will be used to write the .lds file.
	 */
	private String attribute;
	
	/**
	 * The List of files in this section.
	 * In this List are .o and .a files as String. 
	 */
	private List objectList;
	
	/**
	 * A flag which indicates that this sections
	 * object files will be automatically updated.
	 */
	private boolean autoUpdate;
	
	// creation hint values for sections(not for project file output)
	public static final int ATTR_TEXT 	= 0;
	public static final int ATTR_BSS		= 1;
	public static final int ATTR_DATA 	= 2;
	public static final int ATTR_RODATA 	= 3;
	public static final int ATTR_COMM = 4;
	public static final int ATTR_CTORS = 5;
	public static final int ATTR_DTORS = 6;
	public static final int ATTR_GCC_EXCEPTION = 7;
   
	// the indices correspond to the attribute int values
	public static final String ATTR_NAMES[] = { 
			".text", ".bss", ".data", ".rodata", 
			".comm", ".ctors", ".dtors", ".gcc_except_table"
	};
	
	public static final String VEC_SECTION_NAME = ".vector";
	
	// lds output prefix for linkonce
	public static final String LINKONCE = ".gnu.linkonce.";

	
	 /**
	 * Default constructor.
	 * The attribute is set to ".text" by default.
	 * All flags are set to true.
	 */
	public SectionData() {
		// note these defaults!
		sectionName = "";
		attribute = ATTR_NAMES[ATTR_TEXT];
		vmaIsAddress = true;
		vmaSectionName = "";
		LMAequalsVMA = true;
		lmaIsAddress = true;
		lmaSectionName = "";
		autoUpdate = false;
		objectList = new ArrayList();
	}
	
	/**
	 * Constructor.
	 * Sets the section name only.
	 * @param sectionName the name of this section
	 */
	public SectionData(String sectionName) {
		this();
		setSectionName(sectionName);
	}
	
	/**
	 * A handy constructor that sets the section name 
	 * by the given attribute.
	 * @param attributeNo the attribute of this section
	 */
	public SectionData(int attributeNo) {
		this();
		setAttribute(ATTR_NAMES[attributeNo]);
		setSectionName(ATTR_NAMES[attributeNo]);
	}
	
	/**
	 * A handy constructor that sets the section name 
	 * by the given attribute with auto update flag.
	 * @param attributeNo	the attribute of this section
	 * @param autoUpdate	true if this section should be auto update
	 * @since 1.1.1
	 */
	public SectionData(int attributeNo, boolean autoUpdate) {
		this(attributeNo);
		setAutoUpdate(autoUpdate);
	}
	
	/**
	 * Makes a SectionData out of another SectionData
	 * @param original
	 */
	public SectionData(SectionData original) {
		this();
		this.sectionName = original.sectionName;
		if (original.isVmaIsAddress()) {
			this.vma = original.getVma();
			this.vmaIsAddress = true;
		} else {
			this.vmaSectionName = original.getVmaSectionName();
			this.vmaIsAddress = false;
		}
		if (original.isLMAequalsVMA()) {
			this.LMAequalsVMA = true;
		} else {
			this.LMAequalsVMA = false;
			if (original.isLmaIsAddress()) {
				this.lma = original.getLma();
				this.lmaIsAddress = true;
			} else {
				this.lmaSectionName = original.getLmaSectionName();
				this.lmaIsAddress = false;
			}
		}
		
		this.attribute = original.getAttribute();
		this.objectList = new ArrayList(original.getObjectList());
		this.autoUpdate = original.autoUpdate;
	
	}
	
	/**
	 * Creates the labels for the section.
	 * Array0 is the start label, array1 is the end label, array2, 3 is 
	 * the lma label(if LMAequalsVMA == false)
	 * @return the section labels as String[4]
	 */
	public String[] createLabels() {
		sectionLabels = new String[4];
		
		String labelName = null;
		// remove the leading "."
		if (sectionName.startsWith(".")) {
			labelName = sectionName.substring(1);
		} else {
			labelName = sectionName;
		}
		
		sectionLabels[0] = "__START_" + labelName;
		sectionLabels[1] = "__END_" + labelName;
		
		if (!LMAequalsVMA) {
			sectionLabels[2] = "__START_" + labelName + "_lma";
			sectionLabels[3] = "__END_" + labelName + "_lma";
		}
		
		return sectionLabels;
	
	}
	
	/**
	 * Alignment declaratives the linker script file.
	 * 
	 * @return true if this section needs an alignment, otherwise false.
	 * @since 1.1.1
	 */
	public boolean needsAlign() {
		int attributeNo = -1;
		
		// get the attribute index
		for (int i = 0; i < ATTR_NAMES.length; i++) {
			if (attribute.equals(ATTR_NAMES[i])) {
				attributeNo = i;
				break;
			}
		}
		
		// ctors and dtors 
		// need alignment
		switch (attributeNo) {
			case ATTR_CTORS :
			case ATTR_DTORS :
				return true;

			default :
				return false;
		}
	}
	
	
	/**
	 * Returns the attribute String for linkerscript file
	 * output.<br>
	 * 
	 * NOTE:The output should be suspended until the next version.
	 * ex) with c++:".text .text.* .gnu.linkonce.t.*" or normally:".bss"
	 * 
	 * @return	output for linkerscript file
	 * @since 1.1.1
	 */
	public String getAttributeOutputString() {
		int attributeNo = -1;
		
		// get the attribute index
		for (int i = 0; i < ATTR_NAMES.length; i++) {
			if (attribute.equals(ATTR_NAMES[i])) {
				attributeNo = i;
				break;
			}
		}
		
		// some sections need special output
		StringBuffer buf = new StringBuffer(getAttribute());
		switch (attributeNo) {
			// the cases below will remain disabled until the linker has been updated
/*
			case ATTR_TEXT :
			case ATTR_DATA :
			case ATTR_RODATA :
				buf.append(" ");
				buf.append(getAttribute()).append(".*");
				buf.append(" ");
				buf.append(LINKONCE).append(attribute.charAt(1)).append(".*");
				break;
*/	  			
			default :
				break;
		
		}

		// this return value be written in the file
		return new String(buf);
	}
	
	/**
	 * Checks if this section is a default standard section.
	 * Checked by section name.
	 * @return true if this is a default section, otherwise false
	 * @since 1.3.0
	 */
	public boolean isDefaultSection() {
		// section name is a defined one?
		for (int i = 0; i < ATTR_NAMES.length; i++) {
			if (sectionName.equals(ATTR_NAMES[i])) {
				return true;
			}
		}
		return false;
	}
	
	
	// getters and setters
	public String getSectionName() {
		return sectionName;
	}
	public void setSectionName(String sectionName) {
		this.sectionName = sectionName;
		createLabels();
	}
	public boolean isVmaIsAddress() {
		return vmaIsAddress;
	}
	public void setVmaIsAddress(boolean vmaIsAddress) {
		this.vmaIsAddress = vmaIsAddress;
	}
	public long getVma() {
		return vma;
	}
	public void setVma(long vma) {
		this.vma = vma;
		setVmaIsAddress(true);
	}
	public String getVmaSectionName() {
		return vmaSectionName;
	}
	public void setVmaSectionName(String vmaSectionName) {
		this.vmaSectionName = vmaSectionName;
		setVmaIsAddress(false);
	}
	public boolean isLMAequalsVMA() {
		return LMAequalsVMA;
	}
	public void setLMAequalsVMA(boolean aequalsVMA) {
		LMAequalsVMA = aequalsVMA;
	}
	public boolean isLmaIsAddress() {
		return lmaIsAddress;
	}
	public void setLmaIsAddress(boolean lmaIsAddress) {
		this.lmaIsAddress = lmaIsAddress;
	}
	public long getLma() {
		return lma;
	}
	public void setLma(long lma) {
		this.lma = lma;
		setLmaIsAddress(true);
	}
	public String getLmaSectionName() {
		return lmaSectionName;
	}
	public void setLmaSectionName(String lmaSectionName) {
		this.lmaSectionName = lmaSectionName;
		setLmaIsAddress(false);
	}
	public String getAttribute() {
		return attribute;
	}
	public void setAttribute(String attribute) {
		this.attribute = attribute;
	}
	public List getObjectList() {
		return objectList;
	}
	public void setObjectList(List objectList) {
		this.objectList = objectList;
	}
	public boolean isAutoUpdate() {
		return autoUpdate;
	}
	public void setAutoUpdate(boolean autoUpdate) {
		this.autoUpdate = autoUpdate;
	}

	/**
	 * Creates a vector SectionData
	 * attribute is rodata
	 * auto update is false
	 * @param vma			the vma address
	 * @param fileName		the file to allocate to this section(null permitted)
	 * @return	SectionData representing a vector section
	 * @since 1.4.0
	 */
	public static SectionData createVectorSection(long vma, String fileName) {
	
		SectionData vector = new SectionData(SectionData.ATTR_RODATA, false);
		vector.setSectionName(VEC_SECTION_NAME);

		// vector address set as VMA
		vector.setVma(vma);
		if (fileName != null) {
			vector.getObjectList().add(fileName + AbstractFileGenerator.EXT_OBJ);
		}
   	
		return vector;
		
	}
	
	/**
	 * The String expression of this class.
	 * For debug use.
	 */
	public String toString() {
		String str = sectionName;
		str += ":vma=";
		if (vmaIsAddress) {
			str += vma;
		} else {
			str += vmaSectionName;
		}
		str += ":lma/vma=" + LMAequalsVMA;
		if (!LMAequalsVMA) {
			str += ":lma=";
			if (lmaIsAddress) {
				str += lma;
			} else {
				str += lmaSectionName;
			}
		}
		
		if (autoUpdate) {
			str += " AUTO";  
		}
		return str; 
	}


}
