/*
 * Created on 2005/11/17 (refactored from LinkerScriptSetting)
 */
package jp.co.epson.ide.eclipse.gnu33.linkerscript;

import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
  

/**
 * Comparator for sorting the SectionData list.
 * The purpose is to sort the list in vma order.
 * If a section's vma is a reference to another section, 
 * the order depends on the nearest real vma address.
 * @author D.Fujimoto
 * @since 1.2.0
 *
 */
public class VmaSectionComparator implements Comparator {

	static final String SEC1 = "sec1"; //$NON-NLS-1$
	static final String SEC2 = "sec2"; //$NON-NLS-1$
	
	// counter for how many times getNearestVmaAddress() was invoked
	Map counterMap = new HashMap();
	
	// the List of SectionData to search recursively
	private List sections;
	
	/**
	 * Constructor.
	 * Needs the List of SectionData to do recursive searches
	 * @param sections
	 */
	public VmaSectionComparator(List sections) {
		this.sections = sections;
	}
	
	
	/**
	 * Compares SectionData by their address or, the referenced section.
	 * @return -1 when o1 should come before o2 in the list, 
	 * 		1 when o2 should come before o1 in the list, 
	 * 		0 when o1 and o2 are considered equal.
	 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
	 */
	public int compare(Object o1, Object o2) {
		int result = -1;
		
		SectionData sec1 = (SectionData) o1;
		SectionData sec2 = (SectionData) o2;

		// reset counter
		counterMap.put(SEC1, new Integer(0));
		counterMap.put(SEC2, new Integer(0));
		
		long addr1 = -1;
		long addr2 = -1;
		
		if (sec1.isVmaIsAddress()) {
			addr1 = sec1.getVma();
			if (sec2.isVmaIsAddress()) {
				addr2 = sec2.getVma();
				// sec1 and 2 are both addresses
				result = new Long(addr1).compareTo(new Long(addr2));
			} else {
				// find the nearest vma recursively
				addr2 = getNearestVmaAddress(sec2, SEC2);
				// sec1 is an address and sec2 is a reference
				result = new Long(addr1).compareTo(new Long(addr2));
				if (result == 0) {
					// the one with the real address comes first
					result = -1;
				}
			}
		} else {
			addr1 = getNearestVmaAddress(sec1, SEC1);
			if (sec2.isVmaIsAddress()) {
				addr2 = sec2.getVma();
				// sec1 is a reference and sec2 is an address
				result = new Long(addr1).compareTo(new Long(addr2));
				if (result == 0) {
					// the one with the real address comes first
					result = 1;
				}
			} else {
				addr2 = getNearestVmaAddress(sec2, SEC2);
				// sec1 and 2 are both references
				// so decide the order by how far the lma or vma address is
				result = new Long(addr1).compareTo(new Long(addr2));
				if (result == 0) {
					Integer count1 = (Integer) counterMap.get(SEC1);
					Integer count2 = (Integer) counterMap.get(SEC2);
					result = count1.compareTo(count2);
				}
			}
		}
		
		return result;
	  
	}
	
	/**
	 * Returns the given section's nearest real vma address.
	 * If this section's vma is not a real address, this method 
	 * recursively searches until it finds the real address.
	 * <p>
	 * Counts up how many times this method was invoked and stores it in a Map by the 
	 * given key.
	 * @param section the SectionData in question
	 * @param key the key to the counterMap
	 * @return the address or -1 if it cannot find
	 */
	private long getNearestVmaAddress(SectionData section, String key) {

		// count up
		int count = ((Integer) counterMap.get(key)).intValue();
		count++;
		counterMap.put(key, new Integer(count));
		
		// the section must be in the list
		if (!sections.contains(section)) {
			return -1;
		}
		 
		if (section.isVmaIsAddress()) {
			// return back its vma address
			return section.getVma();
		} else {
			// if it references another section, find recursiviely 
			String aSectionName = section.getVmaSectionName();
			SectionData aSection = getSectionByName(aSectionName);

			if (aSection != null) {
				// avoid endless
				if (!section.getSectionName().equals(aSection.getVmaSectionName())) {
					return getNearestVmaAddress(aSection, key);    
				}
			}
		}
		
		// shouldn't come this far
		return -1;

	}
	
	/**
	 * Finds a SectionData from the list by name.
	 * @param sectionName the desired SectionDatas name
	 * @return the SectionData with the name, or null if not found
	 */
	public SectionData getSectionByName(String sectionName) {
		for (int i = 0; i < sections.size(); i++) {
			SectionData aSection = (SectionData) sections.get(i);
			if (aSection.getSectionName().equals(sectionName)) {
				return aSection;
			}
		}
		return null;
	}
	
}
