/*
 * Created on 2004/10/07
 * Modified on 2005/06/17 by D.Fujimoto added method getObjectListForSection()
 * Modified on 2006/07/21 by D.Fujimoto added method fileExistsInProject()
 * Modified on 2007/07/18 by D.Fujimoto added method getSourceFoldersFromProject()
 */
package jp.co.epson.ide.eclipse.gnu33.others;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectNature;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSetting;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSettingFactory;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildOption;

import org.eclipse.cdt.core.CCorePlugin;
import org.eclipse.cdt.core.model.CModelException;
import org.eclipse.cdt.core.model.CoreModel;
import org.eclipse.cdt.core.model.ICProject;
import org.eclipse.cdt.core.model.ISourceRoot;
import org.eclipse.cdt.core.model.ITranslationUnit;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;

/**
 * This class provides various accessibilities to IProjects.
 * Mainly for internal use.
 * @author D.Fujimoto
 *
 */
public class ProjectAccessor {

	/**
	 * The target project we want to access.
	 */
	private IProject project;
	
	/**
	 * List of C source files contained in the project.
	 */
	private List sourceList;
	
	/**
	 * Constructor.
	 * @param project the project to access. must be an open project.
	 */
	public ProjectAccessor(IProject project) {
		this.project = project;
		
	}
	
	/**
	 * Returns a List of IResources which are source files contained in
	 * a C/C++ Projects View. (*.s, *.c, etc)
	 */
	public List getSourceFilesFromProject() {
		
		sourceList = new ArrayList();
		
		try {	
			ICProject cproj = CCorePlugin.getDefault().getCoreModel().create(project);
			ISourceRoot[] sourceRoot = cproj.getSourceRoots();
			for (int i = 0; i < sourceRoot.length; i++) {
				ITranslationUnit[] units = sourceRoot[i].getTranslationUnits();
				for (int j = 0; j < units.length; j++) {
					if (units[j].isSourceUnit()) {
						sourceList.add(units[j].getResource());
					}
				}
			}
			
			cproj = null;
			
			
		} catch (CoreException e) {
			Logger.log(e);
		}

		return sourceList;
	}
	
	/**
	 * Returns a List of IResources which represent source folders
	 * This would contain the project folder itself
	 * @param true to omit project folder
	 * @return List of IResources which represent source folders
	 * @since 1.4.0
	 */
	public List getSourceFoldersFromProject(boolean omitProjectFolder) {
		List folderList = new ArrayList();
		
		try {	
			ICProject cproj = CCorePlugin.getDefault().getCoreModel().create(project);
			ISourceRoot[] sourceRoot = cproj.getSourceRoots();
			for (int i = 0; i < sourceRoot.length; i++) {
				IResource res = sourceRoot[i].getResource();
				if (omitProjectFolder && res.getName().equals(project.getName())) {
					// don't add the project folder
					continue;
				}
				folderList.add(sourceRoot[i].getResource());
			}
			
		} catch (CoreException e) {
			Logger.log(e);
		}
		
		return folderList;
	}
	
	
	/**
	 * Returns a List of String which are assumed to be *.o files after a build.
	 * The String is a project-relative path.
	 * If main.c and boot.s are contained in a project, this method will
	 * return a list containing main.o and boot.o. 
	 * @return List of assumed object file Strings
	 */
	public List getAssumedObjectFiles() {
		List ret = new ArrayList();
		
		if (sourceList == null) {
			getSourceFilesFromProject();
		}
		
		for (int i = 0; i < sourceList.size(); i++) {
			IFile sourceFile = (IFile) sourceList.get(i);
			String fileName = sourceFile.getProjectRelativePath().toString();
			String temp = fileName.substring(0, fileName.lastIndexOf("."));  //$NON-NLS-1$
			temp += ".o";
			ret.add(temp);
			
		}
		
		return ret;
		
	}
	
	/**
	 * Searches the project and returns a list of 
	 * 		asseumed object files,
	 *		libraries set in the build setting.
	 * Used for recreating the object list for a section in a 
	 * linkerscript setting.
	 * The object list will be returned in case insensitive alphabetical order.(except for libs)
	 * @return List of String containing the assumed object files and the libraries
	 * @throws CoreException
	 * @since 1.1.1
	 */
	public List getObjectListForSection() {
		
		List objFiles = getAssumedObjectFiles();
		// sort the object files
		Collections.sort(objFiles, String.CASE_INSENSITIVE_ORDER);
		
  	
		// get the build setting of this project
		Gnu33ProjectDescriptorManager desc = null;
		try {
			desc = Gnu33ProjectDescriptorManager.getDescriptorManager(project);
			
			// get the libraries from the build setting
			BuildSetting build = (BuildSetting) desc.getBuildList().get(0);
			IBuildOption option = build.getLinker().getOption(
					BuildSettingFactory.LD_LIB_OPT_ID);
			String[] userLibs = option.getListValue();
			// make a list of assumed .o files and user libs
			objFiles.addAll(Arrays.asList(userLibs));
			
		} catch (CoreException e) {
			e.printStackTrace();
		}
		
		return objFiles;
	}

	/**
	 * Checks if the file is: contained in a GNU project which is opened, a
	 * translation unit,
	 * 
	 * @return true if the conditions above are checked
	 * @since 1.2.0 
	 * refactored from SourceUnitListener
	 */
	public static boolean isValidTranslationUnit(IFile file) {

		boolean result = false;

		// is it a part of a valid GNU project?
		if (Gnu33ProjectNature.isNatureActive(file.getProject())) {

			// is it a translation unit?
			if (CoreModel.isTranslationUnit(file)) {

				result = true;

			}

		}

		return result;

	}
	
	/**
	 * Checks if the file is contained in a C source folder
	 * 
	 * @param file
	 *			  the file in question
	 * @return true if this file is contained in a C source folder
	 * @since 1.2.0 
	 * refactored from SourceUnitListener
	 */
	public static boolean isContainedInSourceRoot(IFile file) {
		boolean found = false;

		// 
		IContainer cont = file.getParent();
		IPath filePath = cont.getProjectRelativePath();

		ICProject cpj = CoreModel.getDefault().create(file.getProject());

		try {
			ISourceRoot[] roots = cpj.getSourceRoots();
			for (int i = 0; i < roots.length; i++) {
				IPath srcRootPath = roots[i].getResource()
						.getProjectRelativePath();
				if (filePath.toString().equals(srcRootPath.toString())) {
					found = true;
				}

			}

		} catch (CModelException e) {
			e.printStackTrace();
		}

		return found;

	}
	
	
	/**
	 * Checks if the given file exists directly under the project folder
	 * @param fileName the file name to search
	 * @return true if exists, otherwise false
	 * @since 1.3.0
	 */
	public boolean fileExistsInProject(String fileName) {

		// exists directly under the project?
		return project.getFile(fileName).exists();
		
	}

}
