/*
 * Created on 2004/08/03
 *
 */
package jp.co.epson.ide.eclipse.gnu33.parameter;


/**
 * This class represents parameters for memory areas.
 * This class is contained in ParameterSetting.
 * 
 * @author D.Fujimoto
 *
 */
public class ParameterArea implements Comparable {

	// AREA info (combos index)
	public static final int AREA_ROM = 0;
	public static final int AREA_RAM = 1;
	public static final int AREA_IO  = 2;
	public static final int AREA_STACK = 3;
	public static final int AREA_DEFAULT = AREA_ROM;
	
	public static final int ACCESS_BYTE = 0;
	public static final int ACCESS_HALFWORD = 1;
	public static final int ACCESS_WORD = 2;
	public static final int ACCESS_DEFAULT = ACCESS_BYTE;
  
	public static final int WAIT_DEFAULT = 0x07;
	
	/**
	 * The type of this area.
	 * One of AREA_ROM | AREA_RAM | AREA_IO | AREA_STACK
	 */
	private int areaType;
	
	/**
	 * The start address of this area
	 */
	private long startAddress;
	
	/**
	 * The end address of this area
	 */
	private long endAddress;
	
	/**
	 * Wait cycle for memory read
	 * From 0x0 To 0xF
	 */
	private int waitCycleRead;
	
	/**
	 * Wait cycle for memory write
	 * From 0x0 To 0xF 
	 */
	private int waitCycleWrite;
	
	/**
	 * Memory access size
	 * One of ACCESS_BYTE | ACCESS_HALFWORD | ACCESS_WORD
	 */
	private int accessType;
	
	/**
	 * Endian for this area
	 */
	private boolean useBigEndian;
	
	/**
	 * Comments for this area 
	 */
	private String comment = new String();
	
	/**
	 * Create a new ParameterArea.
	 * With some defaults
	 */
	public ParameterArea() {
		super();
		setAreaType(AREA_DEFAULT);
		setWaitCycleRead(WAIT_DEFAULT);
		setWaitCycleWrite(WAIT_DEFAULT);
		setAccessType(ACCESS_DEFAULT);
		
	}

	
	/**
	 * Returns true when the type of this area is AREA_STACK
	 * @return true 
	 */
	public boolean isStackArea() {
		boolean ret = false;
		if (getAreaType() == AREA_STACK) {
			ret = true;
		}
		return ret;
	}
	
	/**
	 * @return the display String for the Table control. 
	 */
	public String getAreaTypeString() {
		String retStr = ""; //$NON-NLS-1$
		switch (areaType) {
			case AREA_RAM:
				retStr = ParameterSettingResources.getString("ParEdit.label.RAM"); //$NON-NLS-1$
				break;
			case AREA_ROM:
				retStr = ParameterSettingResources.getString("ParEdit.label.ROM"); //$NON-NLS-1$
				break;
			case AREA_IO:
				retStr = ParameterSettingResources.getString("ParEdit.label.IO"); //$NON-NLS-1$
				break;
			case AREA_STACK:
				retStr = ParameterSettingResources.getString("ParEdit.label.STACK"); //$NON-NLS-1$
				break;
		}
		return retStr;
		
	}
	
	/**
	 * Converts argument String to area type code
	 * @param typeString the area type String
	 * @return area type code as int 
	 */
	public static int parseAreaTypeInt(String typeString) {
		int ret = 0;
		if (typeString.equals(ParameterSettingResources.getString("ParEdit.label.RAM"))) { //$NON-NLS-1$
			ret = AREA_RAM;
		} else if (typeString.equals(ParameterSettingResources.getString("ParEdit.label.ROM"))) { //$NON-NLS-1$
			ret = AREA_ROM;
		} else if (typeString.equals(ParameterSettingResources.getString("ParEdit.label.IO"))) { //$NON-NLS-1$
			ret = AREA_IO;
		} else if (typeString.equals(ParameterSettingResources.getString("ParEdit.label.STACK"))) { //$NON-NLS-1$
			ret = AREA_STACK;
		}
		return ret;
	}
	
	   
	/**
	 * @param wait wait cycle to be converted to hex String
	 * @return the hex representation of the given value
	 */
	public String getWaitCycleString(int wait) {
		return Integer.toHexString(wait).toUpperCase();
	}
	
	/**
	 * @return the String representation of the given value
	 */
	public String getAccessTypeString() {
		String retStr = ""; //$NON-NLS-1$
		switch (accessType) {
			case ACCESS_BYTE:
				retStr = ParameterSettingResources.getString("ParEdit.label.byte"); //$NON-NLS-1$
				break;
			case ACCESS_HALFWORD:
				retStr = ParameterSettingResources.getString("ParEdit.label.halfword"); //$NON-NLS-1$
				break;
			case ACCESS_WORD:
				retStr = ParameterSettingResources.getString("ParEdit.label.word"); //$NON-NLS-1$
				break;
				
		}
		return retStr;
	}
	
	/**
	 * Used for parameter file generation.
	 * @return the String representation of the given value
	 */
	public String getAccessTypeChar() {
		String retStr = ""; //$NON-NLS-1$
		switch (accessType) {
			case ACCESS_BYTE:
				retStr = ParameterSettingResources.getString("ParEdit.char.byte"); //$NON-NLS-1$
				break;
			case ACCESS_HALFWORD:
				retStr = ParameterSettingResources.getString("ParEdit.char.halfword"); //$NON-NLS-1$
				break;
			case ACCESS_WORD:
				retStr = ParameterSettingResources.getString("ParEdit.char.word"); //$NON-NLS-1$
				break;
				
		}
		return retStr;
	}
	
	/**
	 * Converts argument String to access type code
	 * @param accessString the access type String
	 * @return access type code as int
	 */
	public static int parseAccessTypeInt(String accessString) {
		int ret = 0;
		if (accessString.equals(ParameterSettingResources.getString("ParEdit.label.byte"))) { //$NON-NLS-1$
			ret = ACCESS_BYTE;
		} else if (accessString.equals(ParameterSettingResources.getString("ParEdit.label.halfword"))) { //$NON-NLS-1$
			ret = ACCESS_HALFWORD;
		} else if (accessString.equals(ParameterSettingResources.getString("ParEdit.label.word"))) { //$NON-NLS-1$
			ret = ACCESS_WORD;
		}
		return ret;
			
	}
	
	/**
	 * @return the String representation if big endian should be used
	 */
	public String getUseBigEndianString() {
		return useBigEndian ? "Big" : "" ; //$NON-NLS-2$
	}
	
	/**
	 * Returns this areas type in <code>String</code>
	 */
	public String toString() {

		return getAreaTypeString();
	}
	
	// getters and setters
	public int getAccessType() {
		return accessType;
	}
	public void setAccessType(int accessType) {
		this.accessType = accessType;
	}
	public long getEndAddress() {
		return endAddress;
	}
	public void setEndAddress(long endAddress) {
		this.endAddress = endAddress;
	}
	public long getStartAddress() {
		return startAddress;
	}
	public void setStartAddress(long startAddress) {
		this.startAddress = startAddress;
	}
	public boolean isUseBigEndian() {
		return useBigEndian;
	}
	public void setUseBigEndian(boolean useBigEndian) {
		this.useBigEndian = useBigEndian;
	}
	public int getWaitCycleRead() {
		return waitCycleRead;
	}
	public void setWaitCycleRead(int waitCycleRead) {
		this.waitCycleRead = waitCycleRead;
	}
	public int getWaitCycleWrite() {
		return waitCycleWrite;
	}
	public void setWaitCycleWrite(int waitCycleWrite) {
		this.waitCycleWrite = waitCycleWrite;
	}
	public String getComment() {
		return comment;
	}
	public void setComment(String areaComment) {
		this.comment = areaComment;
	}
	public int getAreaType() {
		return areaType;
	}
	public void setAreaType(int areaType) {
		this.areaType = areaType;
	}


	/**
	 * This is used for sorting ParameterAreas
	 * Note: this class has a natural ordering 
	 * that is inconsistent with the equals() method. 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object o) {
		ParameterArea other = (ParameterArea) o;

		// stack area always come last
		if (this.areaType == AREA_STACK && other.areaType != AREA_STACK) {
			return 1;
		} else if (this.areaType != AREA_STACK && other.areaType == AREA_STACK) {
			return -1;
		}
		
		// sort by address when both areas are/not stack
		if (this.startAddress < other.startAddress && 
				this.endAddress < other.endAddress &&
				this.endAddress < other.startAddress) {
			return -1;
		} else if (this.startAddress > other.startAddress &&
				this.endAddress > other.endAddress &&
				this.startAddress > other.endAddress) {
			return 1;
		} else {
			// should never occur
			return 0;
		}
		
	}
}
