/*
 * Created on 2004/09/14
 * Modified on 2005/08/19 by D.Fujimoto overrided method createControl().
 * Modified on 2006/03/17 by D.Fujimoto pulled up refreshAllPages().
 * Modified on 2006/07/21 by D.Fujimoto added readContentsFromFile(), writeContentsToFile().
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/08/03 by D.Fujimoto added displayErrorDialog().
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.others.Logger;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceNode;
import org.eclipse.jface.preference.IPreferencePage;
import org.eclipse.jface.preference.PreferenceDialog;
import org.eclipse.jface.preference.PreferenceManager;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbenchPropertyPage;
import org.eclipse.ui.dialogs.PropertyPage;

/**
 * Abstract class for all Property Pages related to 
 * a Gnu33 project.
 * Gnu33 Property Pages must extend this class.
 * 
 * @author D.Fujimoto
 *
 */
public abstract class AbstractGnu33PropertyPage extends PropertyPage implements
		IWorkbenchPropertyPage {

	public static final String MSG_UPDATE_FAILED = ResourceProvider.getResourceString("propertyPage.update.failed"); 
	public static final String MSG_PRJ_SETTINGS_FAILED = ResourceProvider.getResourceString("propertyPage.settings.failed");
	
	public static final String DEFAULT_DLG_TITLE = ResourceProvider.getResourceString("default.dialog.title");
	public static final String DEFAULT_DLG_MSG = ResourceProvider.getResourceString("default.dialog.message");
	
	public static final String DEFAULT_BTN_LABEL = ResourceProvider.getResourceString("propertyPage.defaultButton");
	
	/**
	 * The project this page is for.
	 */
	protected IProject project;
	
	/**
	 * The descriptor manager to access the descriptor file.
	 */
	protected Gnu33ProjectDescriptorManager descManager;
   
	/**
	 * Error message for this page.
	 */
	protected String errorMessage = null;
	
	/**
	 * Constructor.
	 * Called by the framework.
	 */
	public AbstractGnu33PropertyPage() {
		super();
	}

	/**
	 * Subclasses should create their own UI contents. 
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	abstract protected Control createContents(Composite parent);

	/**
	 * Indicates whether the Controls of this page has already been created
	 * @return true or false
	 */
	public boolean isContentsCreated() {
		return super.isControlCreated();
		
	}
	
	/**
	 * Get the settings from the project descriptor.
	 * Subclasses may extend.
	 * This method should be called whenever the page should be refreshed.
	 */
	public void getProjectSettings() {
		try {
			project = (IProject) getElement();
			
			descManager = Gnu33ProjectDescriptorManager.getDescriptorManager(project);
			
		} catch (CoreException e) {
			e.printStackTrace();
			Logger.log(e);
			MessageDialog.openError(getShell(), MSG_PRJ_SETTINGS_FAILED, 
					e.getLocalizedMessage());
		}
	};
	

	/**
	 * Refreshes the page contents.
	 * Subclasses must extend their own manner of refreshing 
	 * the Controls in the page.
	 * This method should be called whenever the page should be refreshed.
	 */
	abstract public void refreshPage();
	
	/**
	 * Populate this dialogs settings to the default values.
	 *
	 */
	public void resetToDefaults() {
		getProjectSettings();
		refreshPage();
	}
	
	/** 
	 * Handles default button events. 
	 * Reads the description file over again and refresh the controls.
	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
	 */
	protected void performDefaults() {
		
		boolean doit = MessageDialog.openConfirm(getShell(), 
				DEFAULT_DLG_TITLE, DEFAULT_DLG_MSG);
		if (doit) {
			resetToDefaults();
			super.performDefaults();	
		}
		
	}

	/** 
	 * Checks if this page is valid.
	 * @return false if an error message is set to this page
	 * @see org.eclipse.jface.preference.IPreferencePage#isValid()
	 */
	public boolean isValid() {
	
		if (errorMessage != null) {
			return false;
		}
		return true;
	}
	

	/*
	 * Change the defaults button label
	 * 
	 * @since 1.1.1
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		super.createControl(parent);
		
		Button defaultButton = getDefaultsButton();
		if (defaultButton != null) {
			defaultButton.setText(DEFAULT_BTN_LABEL);
		}
	}
	
	/**
	 * Refresh other Property Pages(that are related to GnuXX project)
	 * in case one page's change affects another.
	 * Note that refresh will be done in PreferenceManager.PRE_ORDER
	 * (order from the top root of the property pages)
	 * @since 1.3.0
	 */
	public void refreshAllPages() {
		// refresh other Property Pages(that are related to GnuXX project)
		PreferenceDialog dialog = (PreferenceDialog) getContainer();
		PreferenceManager man = dialog.getPreferenceManager();
		List list = man.getElements(PreferenceManager.PRE_ORDER);
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			IPreferenceNode node = (IPreferenceNode) iter.next();
			IPreferencePage page = node.getPage();
			if (page != null && page instanceof AbstractGnu33PropertyPage) {
				AbstractGnu33PropertyPage propPage = (AbstractGnu33PropertyPage) page;
				propPage.getProjectSettings();
				// do not try to refresh it unless the controls are created
				if (propPage.isContentsCreated()) {
					propPage.refreshPage();    
				}
				
			}
		}
	}
	
	
	/**
	 * Read the given file and contents and returns.
	 * This will be used when the Property Page has to read the 
	 * associated file.
	 * @return the contents of the file or null when an error occured
	 * @since 1.3.0
	 */
	protected String readContentsFromFile(String fileName) {
		String contents = "";
		
		try {
			InputStream inStream = project.getFile(fileName).getContents();
			BufferedReader br = new BufferedReader(new InputStreamReader(inStream));
			String line = null;
			while ((line = br.readLine()) != null) {
				contents += line + "\n";
			}
			br.close();
			inStream.close();

		} catch (IOException e) {
			e.printStackTrace();
			contents = null;
		} catch (CoreException e) {
			e.printStackTrace();
			contents = null;
		}
		
		return contents;
	}
	
	/**
	 * Writes the contents to the given file.
	 * The file will be overwritten if it already exists.
	 * 
	 * @param fileName	the file name to write to
	 * @param contents the contents to write
	 * @return	true if succeeded, false otherwise
	 * @since 1.3.0
	 */
	protected boolean writeContentsToFile(String fileName, String contents) {
		boolean succeeded = true;
		try {
			ByteArrayInputStream stream = new ByteArrayInputStream(contents.getBytes());
			IFile file = project.getFile(fileName);
			if (!file.exists()) {
				file.create(stream, true, null);
			} else {
				file.setContents(stream, true, false, null);	
			}
			stream.close();
		} catch (CoreException e) {
			succeeded = false;
		} catch (IOException e) {
			e.printStackTrace();
			succeeded = false;
		}
		return succeeded;
	}
	
	/**
	 * Displays the message in a dialog when an error occurs in one of
	 * the property pages  
	 * @param message	the error message
	 * @since 1.4.0
	 */
	protected void displayErrorDialog(String title, String message) {
		MessageDialog.openError(getShell(), title, message);
	}
	
	
}
