/*
 * 쐬: 2005/08/24
 *
 */
package gnu33dll;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.StringTokenizer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.OSMonitorPlugin;

import org.eclipse.jface.preference.IPreferenceStore;


/**
 * JNI interface for OS Monitor.
 * @author D.Fujimoto
 *
 */
public class OSMonitorConnection extends Thread {

	/**
	 *  interval to wait before the next trial (msec)
	 */
	public static final int INTERVAL_MSEC;

	/**
	 * Max timeout seconds.<br>
	 * The loop will continue for this amount of time.
	 */
	public static final int TIMEOUT_MSEC;
	
	/**
	 * default status code
	 */ 
	public static final int RET_INTERNAL_ERR = -1;
	// status codes must be identical with those in the library
	public static final int RET_OK = 0;
	public static final int RET_NO_TARGET_ERR = 1;				// not connect target yet
	public static final int RET_INVALID_COMMAND_ERR = 2;			// invalid command
	public static final int RET_COMMAND_SEQ_ERR = 3;				// command sequence error
	public static final int RET_OVER_BUFFER_MAX_ERR = 4;			// over read/write buffer size
	public static final int RET_OVER_SYMBOL_MAX_ERR = 5;			// over symbol name length
	public static final int RET_NO_COMMAND = 6;					// no command from IDE yet
	public static final int RET_NO_READ_BUFFER = 7;				// no read buffer from GDB yet
	public static final int RET_WAIT_SYMBOL = 8;					// no symbol name from GDB yet
	public static final int RET_NO_SYMBOL = 9;					// failed conversion address to symbol
	public static final int RET_WAIT_IL = 10;					// no %psr IL bit from GDB yet
	public static final int RET_TARGET_RUNNING = 11;				// the target is still running

	/**
	 * default interrupt level
	 */
	public static final int IL_UNKOWN = -1;						// unknown interrupt level
	
	// command constants for setCommand()
	public static final int CMD_READ = 0;						// read memory
	public static final int CMD_WRITE = 1;						// write memory
	public static final int CMD_GET_SYMBOL = 2;					// get symbol from address
	public static final int CMD_GET_IL = 3;						// get debugger interrupt level

	/**
	 * max bytes for reading/writing.
	 * should not exceed the maximum of int.
	 */
	public static final int BUFFER_MAX = 1024;
	
	/**
	 * max chars for getSymbol()
	 */
	public static final int ADDRESS_SYMBOL_MAX = 300; 

	
	/**
	 * The connection library name.<br>
	 * For Windows:gnu33ide.dll/For Linux:libgnu33ide.so
	 */
	public static final String LIB_NAME;
	
	/**
	 * the singleton instance
	 */
	private static OSMonitorConnection connection;
	

	static {
		IPreferenceStore store = OSMonitorPlugin.getDefault().getPreferenceStore();

		LIB_NAME = store.getString("connection.libName");
		TIMEOUT_MSEC = store.getInt("connection.timeout");
		INTERVAL_MSEC = store.getInt("retry.interval");
		// for standalone debugging
//		LIB_NAME = "gnu33ide.dll";
//		TIMEOUT_MSEC = 5000;
//		INTERVAL_MSEC = 50;		
		
		// Load the library
		System.load(OSMonitorPlugin.getToolsPath().append(LIB_NAME).toOSString());
		// for standalone debugging
//		System.load("c:\\gnu33\\" + LIB_NAME);

		if (OSMonitorPlugin.getDefault().isDebugging()) {
			System.out.println("Loaded library");
		}
	}
	
	/**
	 * Creates a singleton instance.
	 * 
	 * @return	OSMonitorConnection
	 */
	public static OSMonitorConnection getConnection() {
		if (connection == null) {
			connection = new OSMonitorConnection();
		}
		return connection;
	}
	
	
	/**
	 * For command line debugging.<br>
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		
		System.out.println("Start");	//$NON-NLS-1$
		
		// make a connection
		OSMonitorConnection.getConnection();
		
		// enter "q" into command line to quit
		BufferedReader con = getInput();
		try {
			
			String line = null;
			StringTokenizer tok = null;
			String cmd = null;
			String[] cmdArgs = null;
			while ((line = con.readLine()) != null) {

				// tokenize the command line
				tok = new StringTokenizer(line);
				cmd = "";
				if (tok.countTokens() > 0) {
					cmd = tok.nextToken();
				}
				
				int index = 0;
				cmdArgs = new String[tok.countTokens()];
				while (tok.hasMoreTokens()) {
					cmdArgs[index++] = tok.nextToken();
				}
			
				if (cmd.equals("q")) {	//$NON-NLS-1$
					System.out.println("Terminated");	//$NON-NLS-1$
					System.exit(0);
				}
				
				try {
					processCommandLine(cmd, cmdArgs);
				} catch (OSMonitorConnectionException e) {
					System.out.println(e.getCauseFlag());
				}

			}
		} catch(IOException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * For command line debugging.<br>
	 * 
	 * @param cmd
	 * @param cmdArgs
	 * @throws OSMonitorConnectionException
	 */
	private static void processCommandLine(String cmd, String[] cmdArgs) throws OSMonitorConnectionException {
	
		if (cmd.equals("w")) {	//$NON-NLS-1$
			/** write **/
			/*
			 * Format:
			 * >w SYMBOL			w main
			 * >w HEX_ADDRESS		w 600000
			 **/
			
			int[] writeData = {0x1, 0x2, 0x64, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF, 0xFF};

			String symbol = null;
			long address = 0x600000;
			
			if (cmdArgs.length != 0) {
				try {
					address = Long.parseLong(cmdArgs[0], 16);
				} catch (NumberFormatException e) {
					symbol = cmdArgs[0];
					address = 0;
				}
			}
			
			/** returns OK or NO_TARGET, otherwise throws an exception */
			int ret = connection.write(symbol, address, writeData);
			
		} else if (cmd.equals("r")) {	//$NON-NLS-1$
			/** read **/
			/*
			 * Format:
			 * >r BYTES SYMBOL			r 16 main
			 * >r BYTES HEX_ADDRESS		r 16 600000
			 */
			
			String symbol = null;
			long address = 0x600000;
			int size = 16;
			
			if (cmdArgs.length >= 2) {
				try {
					size = Integer.parseInt(cmdArgs[0]);
				} catch (NumberFormatException e) {	}
				try {
					address = Long.parseLong(cmdArgs[1], 16);
				} catch(NumberFormatException e) {
					symbol = cmdArgs[1];
					address = 0;
				}
			}
			
			char[] readData = new char[size];
			
			/** returns OK or NO_TARGET, otherwise throws an exception **/
			int ret = connection.read(symbol, address, readData);
			
			if (ret == RET_OK) {
				connection.showData(readData);
			}
			
		} else if (cmd.equals("c")) {	//$NON-NLS-1$
			/** counter **/
			/*
			 * Format:
			 * >c
			 */
			
			/** directly invoke the native method **/
			int counter = connection.getGoCounter();
			
			System.out.println("counter = " + counter);
			
		} else if (cmd.equals("s")) {	//$NON-NLS-1$
			/** symbol **/
			/*
			 * Format:
			 * >s HEX_ADDRESS			s C00000
			 */
			
			long address = 0xc00004;
			
			if (cmdArgs.length > 0) {
				try {
					address = Long.parseLong(cmdArgs[0], 16);
				} catch (NumberFormatException e) {}
			}
			
			/** returns the symbol when OK, null at timeout, otherwise throws an exception **/			
			String symbol = connection.getSymbolFromAddress(address);
			
			if (symbol == null) {
				System.out.println("Cannot get symbol at address " + Long.toHexString(address));
			} else {
				System.out.println("Symbol = " + symbol + " at " + Long.toHexString(address));
			}
			
		} else if (cmd.equals("i")) {	//$NON-NLS-1$
			/** interrupt level **/
			/*
			 * Format:
			 * >i
			 */
			
			/** returns the level when OK, -1 at timeout, otherwise throws an exception **/
			int level = connection.getInterruptLevel();
			
			if (level == IL_UNKOWN) {
				System.out.println("Cannot get interrupt level");
			} else {
				System.out.println("IL = " + level);
			}
		} 
	
	
	}
	
	
	
	/**
	 * Gets the console input buffer.<br>
	 * 
	 * @return the console BufferedReader
	 */
	private static BufferedReader getInput() {
		BufferedReader con = new BufferedReader(new InputStreamReader(System.in));
		return con;
	}
	
	/**
	 * Checks whether the current time has not exceeded the timeout value.
	 * @param startTime	the base time
	 * @return	true if not timed out, false if timed out.
	 */
	private static boolean notTimedOut(long startTime) {
		
		return (System.currentTimeMillis() - startTime) < TIMEOUT_MSEC;
	
	}
	
	/**
	 * Constructor.
	 * Instantiation is not allowed outside of this class.
	 */
	private OSMonitorConnection() {}
	

	/**
	 * Reads data from the dll.
	 * Timeout will not throw an exception.
	 * 
	 * @param addressSymbol	the address symbol to read from
	 * @param address			the address to read from (0x0-0xFFFFFFFF)
	 * @param readData			the data will be stored here
	 * 							the length of this array is the total length
	 * 							that should be retreived 
	 * @return	result flag		RET_OK				succeeded
	 * 							RET_NO_TARGET		target not connected
	 * 							RET_INTERNAL_ERR	unknown error
	 * @throws OSMonitorConnectionException on other errors
	 * 
	 * If addressSymbol is null or empty, address will be used.
	 * <p>
	 * This routine contains a complicated loop dealing with reading data 
	 * when the total desired size is over the maximum(BUFFER_MAX) value of the connection.
	 * When the addressSymbol is passed on, the size must not exceed BUFFER_MAX.
	 * </p>
	 */
	public int read(String addressSymbol, long address, char[] readData) throws OSMonitorConnectionException {

		int result = RET_INTERNAL_ERR;
		int size = readData.length;
		
		// check which address should be used
		if (addressSymbol == null) {
			addressSymbol = "";
		}
		if (addressSymbol.length() != 0) {
			address = 0;
		}
		

		// calculate the required loops count
		int loops = (size + BUFFER_MAX - 1) / BUFFER_MAX;
		
		long offsetAddress = 0;						// address to read from
		int readSize = 0;							// size to read in one loop
		int offset = 0;								// the offset and array index to copy to
		
		char[] readBuf = new char[BUFFER_MAX];		// the buffer for one loop
		
		for (int i = 0; i < loops; i++) {
			// prepare the parameters for the loop 
			offset = BUFFER_MAX * i;
			offsetAddress = address + offset;
			readSize = loops - 1 == i ? (size - 1) % BUFFER_MAX + 1 : BUFFER_MAX;	// last loop or not
			
			// invoke setCommand() and then getMem()
			result = setCommand(CMD_READ, addressSymbol, offsetAddress, readSize, null);
			switch (result) {
				case RET_OK :
//					System.out.println("Succeeded setCommand() read");
					
					// the loop should break after timeout
					long startTime = System.currentTimeMillis();
					while (notTimedOut(startTime)) {
						
						// read from the memory
						result = getMem(readSize, readBuf);
						
						if (result == RET_OK) {
							// copy the buffer to the return array
							System.arraycopy(readBuf, 0, readData, offset, readSize);
							break;
						} else if (result == RET_NO_READ_BUFFER) {
							
							// wait a while before the next loop 
							try {
								sleep(INTERVAL_MSEC);
							} catch (InterruptedException e) {
								e.printStackTrace();
								return RET_INTERNAL_ERR;
							}
							
						} else {
							// error occured
							break;
						}
						
					}
					
					if (result == RET_OK) {
						break;
					} else if (result == RET_NO_READ_BUFFER) {
						System.out.println("Timed out");
					} else {
						System.out.println("Failed getMem()");
						throw new OSMonitorConnectionException(result);
					}
					
					break;
					
				case RET_NO_TARGET_ERR :
					System.out.println("Not connected to target");
					break;
				
				case RET_TARGET_RUNNING :
					System.out.println("The target is still running");
					break;
					
				default :
					System.out.println("Failed setCommand() read");
					throw new OSMonitorConnectionException(result);
			
			}
			
		}
	
		return result;
		
	}
	
	
	/**
	 * Writes data to the dll.
	 * 
	 * @param	addressSymbol	the address symbol to write to
	 * @param	address			the address to write to
	 * @param	writeData		the data to write
	 * @return	result flag	
	 * 							RET_OK				succeeded
	 * 							RET_NO_TARGET_ERR	target not connected
	 * 							RET_INTERNAL_ERR	unknown error
	 * @throws	OSMonitorConnectionException	on other errors
	 * 						
	 * If addressSymbol is null or empty, address will be used
	 */
	public int write(String addressSymbol, long address, int[] writeData) throws OSMonitorConnectionException {

		int result = RET_INTERNAL_ERR;
		
		// check which address should be used
		if (addressSymbol == null) {
			addressSymbol = "";
		}
		if (addressSymbol.length() != 0) {
			address = 0;
		}
		
		// invoke directly setCommand()
		result = setCommand(CMD_WRITE, addressSymbol, address, writeData.length, writeData); 
		
		switch (result) {
			case RET_OK :
				System.out.println("Succeeded setCommand() write");
				break;
			case RET_NO_TARGET_ERR :
				System.out.println("Not connected to target");
				break;
			case RET_TARGET_RUNNING :
				System.out.println("The target is still running");
				break;
			default :
				System.out.println("Failed setCommand() write");
				throw new OSMonitorConnectionException(result);
		
		}
		
		return result;
		
	}
	
	
	/**
	 * Returns the symbol at the given address.
	 * Timeout will not throw an exception.
	 * 
	 * @param address	the address where a symbol should be defined.
	 * @return 		the symbol String. Returns null if timed out. 
	 * @throws OSMonitorConnectionException	when the symbol does not exist at the address
	 */
	public String getSymbolFromAddress(long address) throws OSMonitorConnectionException {
		
		int result = RET_INTERNAL_ERR;
		char[] symbolBuffer = new char[ADDRESS_SYMBOL_MAX];		// the symbol
		
		// first, send the command
		result = setSymbolGetCommand(address);
		switch (result) {
			case RET_OK :
				
				// the loop should break after timeout
				long startTime = System.currentTimeMillis();
				while (notTimedOut(startTime)) {
					
					// retreive the symbol
					result = getSymbol(symbolBuffer);
					
					if (result == RET_OK) {
						return new String(symbolBuffer).trim();
					} else if (result == RET_WAIT_SYMBOL) {
						
						// wait a while before the next loop 
						try {
							sleep(INTERVAL_MSEC);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
						
					} else {
						// error occured
						break;
					}
					
				}
				
				if (result == RET_NO_READ_BUFFER) {
					System.out.println("Timed out");
				} else {
					System.out.println("Failed getSymbolFromAddress()");
					return null;
//					throw new OSMonitorConnectionException(result);
				}
				
				break;
			default :
				throw new OSMonitorConnectionException(result);
		}
		
		// failure
		return null;
	}
	
	/**
	 * Returns the debugger's interrupt level.
	 * Timeout will not throw an exception.
	 * 
	 * @return	interrupt level. Returns -1 when unable to retreive.
	 * @throws OSMonitorConnectionException
	 */
	public int getInterruptLevel() throws OSMonitorConnectionException {
		int result = RET_INTERNAL_ERR;
		int[] level = { IL_UNKOWN };					// the interrupt level(1 length array)
		
		// first, send the command
		result = setInterruptLevelGetCommand();
		switch (result) {
			case RET_OK :
				
				// the loop should break after timeout
				long startTime = System.currentTimeMillis();
				while (notTimedOut(startTime)) {
					// retreive the interrupt level
					result = getInterruptLevel(level);

					if (result == RET_OK) {
						return level[0];
					} else if (result == RET_WAIT_IL) {
						
						// wait a while before the next loop 
						try {
							sleep(INTERVAL_MSEC);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					} else {
						// error occured
						break;
					}
				}

				if (result == RET_NO_READ_BUFFER) {
					System.out.println("Timed out");
				} else {
					System.out.println("Failed getInterruptLevel()");
					throw new OSMonitorConnectionException(result);
				}
				
				break;
			default :
				throw new OSMonitorConnectionException(result);
		}
		
		return IL_UNKOWN;
	}

	/**
	 * Returns the go counter value.
	 * @return	counter value
	 */
	public int getCounter() {
		return getGoCounter();
	}
	
	/**
	 * Checks if the connection is busy(currently running)
	 * @return	true if busy, otherwise false
	 */
	public boolean isConnectionBusy() {
		return getConnectedState() == RET_TARGET_RUNNING;
	}
	
	/**
	 * Checks if the connection is connected to an OS and ready
	 * @return
	 */
	public boolean isConnectionOk() {
		return getConnectedState() == RET_OK;
	}
	
	/**
	 * Returns the session key.
	 * @return
	 */
	public long sessionKey() {
		return getSessionKey();
	}
	
	/**
	 * For debug use.
	 * 
	 * @param data
	 */
	public void showData(char[] data) {
		
		System.out.println("Java***********************************");
		for (int i = 0; i < data.length; i++) {
			System.out.println("DATA[" + i + "] = " + (int) data[i]);
		}
	
	}
	
//////////////////////////////////////////////////////////////////////
//	Native interfaces	
//  IMPORTANT:If any of these interfaces change, rebuild the jni header file. 	
	
	/**
	 * Native interface.<br>
	 * Sends a read/write command.
	 * 
	 * @param command		command(one of CMD_READ | CMD_WRITE)
	 * @param addrSymbol	the address symbol in String. 
	 * 						null is not allowed.(pass an empty string) 
	 * @param address		start address(if addrSymbol is not specified)
	 * 						this value will be used if addrSymbol is empty
	 * @param size			dump size in bytes
	 * @param writeData 	data to write to when command is CMD_WRITE
	 * @return RET_OK or RET_ERR
	 */
	private native int setCommand(int command, String addrSymbol, long address, int size, int[] writeData);

	
	/**
	 * Native interface.<br>
	 * Reads the memory into the char array.
	 * 
	 * @param size		size to read in bytes
	 * @param data		the data buffer to receive
	 * @return	RET_OK or RET_NO_READ_BUFFER
	 */
	private native int getMem(int size, char[] data);

	
	
	/**
	 * Native interface.<br>
	 * Receives the go-counter from the debugger.
	 * 
	 * @return	counter		from 0 to 0xFFFF
	 */
	private native int getGoCounter();
	
	/**
	 * Native interface
	 * Sends a command for retreiving the symbol from an address.
	 * Should be called before getSymbol().
	 * 
	 * @param address		the address we want the symbol from 
	 * @return	RET_OK | RET_NO_TARGET_ERR | RET_COMMAND_SEQ_ERR
	 */
	private native int setSymbolGetCommand(long address);
	
	/**
	 * Native interface.<br>
	 * Reads the symbol at the address set in setSymbolGetCommand().
	 * Should be called after setSymbolGetCommand().
	 * 
	 * @param symbol			the symbol will be stored in here (in ASCII)
	 * @return	RET_OK			symbol retreived
	 * 			RET_NO_SYMBOL	symbol does not exist
	 * 			RET_WAIT_SYMBOL	have not retreived symbol yet	
	 */
	private native int getSymbol(char[] symbol);
	
	/**
	 * Native interface.<br>
	 * Sends a command for retreiving the interrupt level.
	 * Should be called before getInterruptLevel()
	 * 
	 * @return	RET_OK | RET_NO_TARGET_ERR | RET_COMMAND_SEQ_ERROR
	 */
	private native int setInterruptLevelGetCommand();
	
	/**
	 * Native interface.<br>
	 * Gets the debugger's current interrupt level.
	 * 
	 * 
	 * @param	level		the level will be stored in here
	 * @return	RET_OK		succeeded
	 * 			RET_WAIT_IL	have not retreived interrupt level yet
	 */
	private native int getInterruptLevel(int[] level);

	/**
	 * Native interface.<br>
	 * Gets the debugger's connected state.
	 * 
	 * @return RET_OK		ok to send/receive data
	 * 			RET_NO_TARGET_ERR	debugger is not connected
	 * 			RET_TARGET_RUNNING	debugger is running(busy)
	 */
	private native int getConnectedState();
	
	/**
	 * Native interface.<br>
	 * Resets the command update flag when read/write terminates
	 * in error state.
	 * This may need to be called upon time out at read/write operations.
	 * 
	 * @return RET_OK (always)
	 */
	private native int clearCommand();

	/**
	 * Native interface.<br>
	 * Reads the value of the session key and returns it.
	 * @return	the value may be 0
	 */ 
	private native long getSessionKey();
	
}
