/*
 * 쐬: 2005/09/22
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps;

import gnu33dll.OSMonitorConnection;
import gnu33dll.OSMonitorConnectionException;

import java.nio.CharBuffer;
import java.util.ArrayList;
import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.DataWithParser;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.OSMonitorPlugin;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.IOSMonitorController;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.command.MonitorCommand;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.ToppersMonitorInput;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.DataQueueData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.EventFlagData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.INamedProcess;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.TaskData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.TaskManagementData;


/**
 * Controller for Toppers/JSP OS.<br>
 * 
 * This class has the following functions:<br>
 * 
 * Holds the data model.<br>
 * Handles events (from the UI).<br>
 * Retreives data from the debugger.<br>
 * Parses data and fills the data model.<br>
 * Provides access to the data model.<br>
 * 
 * The reference to this object should only be held at
 * the plugin class.
 * 
 * @author D.Fujimoto
 *
 */
public class ToppersMonitorController implements IOSMonitorController {

	// the controller's target OS. used when the OS has this name.
	private static final String TARGET_OS_NAME = "TOPPERS/JSP";	//$NON-NLS-1$
	
	/**
	 * Info for address and size for retreiving data.
	 */
	private ToppersObjectTable objTable;
	
	/**
	 * Info for system specific settings.
	 */
	private ToppersSystemInfo sysInfo;
	
	/**
	 * Data container.<br>
	 * This will be created here and passed to the UI viewer.
	 */
	private ToppersMonitorInput input;
	
	/**
	 * Flag indicating that this controller is
	 * properly initialized.
	 */
	private boolean initialized;

	/**
	 * Counters for each UI request.
	 * The indices correspond to the request type in 
	 * handleUIRequest().
	 * The values will be stored from 0x0 - 0xFFFF
	 */
	private int[] requestCounters = new int[11];  
	
	/**
	 * Session value to determine whether initialization to
	 * this controller is necessary. 
	 */
	private long sessionKey; 
	
					
	/**
	 * Constructor.
	 * This constuctor should not throw an exception.
	 */
	public ToppersMonitorController() {
		
		// initialize the address table
		objTable = new ToppersObjectTable();
		
		// initialize the system info
		sysInfo = new ToppersSystemInfo();
		
	}
	
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.IOSMonitorController#getTargetOSName()
	 */
	public String getTargetOSName() {
		return TARGET_OS_NAME;
	}
		
	/**
	 * Initializing operation.<br>
	 * Reads the object table.
	 * If the object table is read, creates the data models.
	 *
	 */
	public void init() {
		
		// counter reset
		resetCounters();
		
		// read the object table(this is done only once)
		readFromSymbol(ToppersObjectTable.ADDRESS_SYMBOL, ToppersObjectTable.READ_SIZE, objTable);

		if (objTable.isLoaded()) {
			
			// create the input objects
			input = new ToppersMonitorInput(objTable.getObjectInfoTable());

			initialized = true;
		} else {
			initialized = false;
		}
	
	}
	
	/**
	 * Initialize given a session key.
	 * The key will be tested next time this controller may need 
	 * re-initialization.
	 */
	public void init(long sessionKey) {
		// store the session key
		this.sessionKey = sessionKey;
		init();
	}

	/**
	 * Check if this controller is properly initialized.
	 * @see ToppersMonitorController#init()
	 * @return	true of false.
	 */
	public boolean isInitialized() {
		return initialized;
	}
	
	/**
	 * Interface for retreiving the monitor input.
	 * @return		the monitor input as Object. May be null. 
	 */
	public Object getMonitorInput() {
		return input;
	}
	
	/**
	 * Returns the session key.
	 */
	public long getSessionKey() {
		return sessionKey;
	}
	
	/**
	 * Resets the initialized state of this controller.
	 * 
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.IOSMonitorController#invalidate()
	 */
	public void invalidate() {
		// flag OFF
		initialized = false;
		
		// discard the input
		input = null;
	}
	
	
	/**
	 * Reads data from the given symbol with the given size and
	 * fills the data object
	 * 
	 * @param 	symbol	the address symbol to read from
	 * @param	size	the size to read(in bytes)
	 * @param	data	the data (with a parser) that will be filled
	 * @return	 0 if success, other if failed(depending on OSMonitorConnection)
	 */
	public int readFromSymbol(String symbol, int size, DataWithParser data) {
		int result = 0;
		boolean loaded = false;
		char[] readData = new char[size];
		OSMonitorConnection conn;
		
		try {
			// connect with debugger
			conn = OSMonitorConnection.getConnection();
			
			result = conn.read(symbol, 0, readData);
			
			if (result == OSMonitorConnection.RET_OK) {
				
				data.parseData(readData);
				loaded = true;
			
			} else if (result == OSMonitorConnection.RET_NO_TARGET_ERR) {
				// not connected with debugger
			} else if (result == OSMonitorConnection.RET_TARGET_RUNNING) {
				// target is still running
				// use the last state of controller
				loaded = data.isLoaded();
			}
			
		} catch (OSMonitorConnectionException e) {
			e.printStackTrace();
			result = e.getCauseFlag();
		} finally {
			conn = null;
		}
		
		data.setLoaded(loaded);
	
		return result;
	
	}
	
	
	/**
	 * Reads data by the given type(address, size) and returns it
	 * in char array format.
	 * @param info		contains the address, size and count  
	 * @return 		the read data, or null if failed
	 */
	private char[] readDataByType(KernelObjectInfo info) {

		char[] readData = new char[info.dataCount * info.dataSize];
		OSMonitorConnection conn;
		
		// do not read when count is zero
		if (info.dataCount == 0) {
			return null;
		}
		
		try {
			// connect with debugger
			conn = OSMonitorConnection.getConnection();
			
			int result = conn.read(null, info.dataAddress, readData);
				
			if (result == OSMonitorConnection.RET_OK)  {
				return readData;
			} else if (result == OSMonitorConnection.RET_NO_TARGET_ERR) {
				// not connected with debugger
			}
			
		} catch (OSMonitorConnectionException e) {
			e.printStackTrace();
		} finally {
			conn = null;
		}
		
		return null;
	
	}
	
	/**
	 * Reads data provided with the info(address, size) contained in
	 * KernelObjectInfo and sets the data in the given AbstractDataModel array.
	 * The data will be set by calling the AbstractDataModel's parseData().
	 * 
	 * The data count in KernelObjectInfo and the length of the array must 
	 * be identical.
	 * 
	 * @param objInfo		info on how to read the data 
	 * @param models		the models to be filled
	 * @return		true if reading and parsing succeeded, otherwise false.
	 */
	private boolean readAndParse(KernelObjectInfo objInfo, AbstractDataModel[] models) {
		
		boolean result = false;
		char[] readData = null;
		char[] bufData = null;
		CharBuffer cb = null;
		
		
		// read data (address and size is in KernelObjectInfo)
		readData = readDataByType(objInfo);
		if (readData != null) {
			
			// slice out 1 data structure 
			cb = CharBuffer.wrap(readData);
			bufData = new char[objInfo.dataSize];			
			
			for (int i = 0; i < models.length; i++) {
				cb.get(bufData);
				
				// let the model parse it
				models[i].parseData(objInfo.type, bufData);
			}
			
			result = true;
		}

		
		return result;
	}
	
	/**
	 * Reads the symbol name from the given address.
	 * @return the symbol name of null if not found.
	 */
	private String readSymbolName(long address) {
		String symbol = null;
		OSMonitorConnection conn;
		
		try {
			// connect with debugger
			conn = OSMonitorConnection.getConnection();
			
			symbol = conn.getSymbolFromAddress(address);
			
		} catch (OSMonitorConnectionException e) {
			e.printStackTrace();
		} finally {
			conn = null;
		}
		
		return symbol;
	
	}
	
	
	/**
	 * Reads the interrupt level
	 * @return interrupt level, -1 if failed.
	 */
	private int readInterruptLevel() {
		
		OSMonitorConnection conn;
		
		try {
			// connect with debugger
			conn = OSMonitorConnection.getConnection();
			
			return conn.getInterruptLevel();
		} catch (OSMonitorConnectionException e) {
			e.printStackTrace();
		} finally {
			conn = null;
		}
		
		return -1;
	
	}
	
	/**
	 * Reads the request counter from the library.
	 * @return
	 */
	private int readCounter() {
		
		OSMonitorConnection conn;
		
		try {
			// connect with debugger
			conn = OSMonitorConnection.getConnection();
			
			return conn.getCounter();

		} finally {
			conn = null;
		}
		
	}

	/**
	 * Reads the wait info for event flags and sets the result to the 
	 * associated TaskData.
	 * @param flagData		event flag data that may have the queue indices
	 * @param tasks		the TaskData array(wait info will be set here)
	 */
	private void readWaitInfo(IQueue[] flagData, TaskData[] tasks) {
		
		// read the wait info
		KernelObjectInfo waitInfo = new KernelObjectInfo();
		waitInfo.type = EventFlagData.TYPE_WAIT_INFO;
		waitInfo.dataCount = 1;
		waitInfo.dataSize = 20;
 
		
		for (int i = 0; i < flagData.length; i++) {
			// get the indices for the TaskData[]
			int[] indices = flagData[i].getQueueIndices(IQueue.DEFAULT_TYPE);

			// the length will be 0 or 1 
			if (indices.length != 0) {
				// set the address to read from
				int taskIndex = indices[0];
				waitInfo.dataAddress = tasks[taskIndex].getWaitInfoAddress();
				
				// read the wait info and set it to the TaskData
				readAndParse(waitInfo, new AbstractDataModel[] { (AbstractDataModel) flagData[i] });
				
			}
			
		}

	}
	
	/**
	 * Reads and fills the TaskManagementData.<br>
	 * Also, sets the locked status.
	 * 
	 */
	public void getMngData() {
		
		KernelObjectInfo[] objInfo = objTable.getObjectInfoTable();
		
		// these object infos will be used 
		KernelObjectInfo intNest = objInfo[ToppersObjectTable.TYPE_INTERRUPT_NEST];
		KernelObjectInfo dispFlg = objInfo[ToppersObjectTable.TYPE_DISPATCH_FLAG];
		KernelObjectInfo runTask = objInfo[ToppersObjectTable.TYPE_RUN_TASK_ADDR];
		KernelObjectInfo rdyTask = objInfo[ToppersObjectTable.TYPE_RDY_TASK_ADDR];
		KernelObjectInfo sysTime = objInfo[ToppersObjectTable.TYPE_SYSTIME];
		
		// create data model
		int dataCount = 1;
		TaskManagementData mng = input.getTaskManagementData();
		AbstractDataModel[] models = new AbstractDataModel[] { mng };

		// read data
		readAndParse(intNest, models);
		readAndParse(dispFlg, models);
		readAndParse(runTask, models);
		readAndParse(rdyTask, models);
		readAndParse(sysTime, models);

		// judge the lock status
		int intLevel = readInterruptLevel();
		((TaskManagementData) models[0]).setLocked(intLevel >= sysInfo.getCpuLockLevel()); 
		
		if (models != null && models.length > 0) {
			input.setTaskManagementData(models);
		}
		
	}


	/**
	 * Reads and fills the TaskData[]<br>
	 * Also, sets the control block addresses and 
	 * retreives the task names for each task. 
	 * 
	 * 
	 */
	public void getTaskData() {
		
		KernelObjectInfo[] objInfo = objTable.getObjectInfoTable();
		
		// these object infos will be used 
		KernelObjectInfo cntlInfo = objInfo[ToppersObjectTable.TYPE_TASK_CTRL];
		KernelObjectInfo initInfo = objInfo[ToppersObjectTable.TYPE_TASK_INIT];
		
		// get data model
		AbstractDataModel[] models = input.getTaskData();
		
		// read data
		readAndParse(cntlInfo, models);
		readAndParse(initInfo, models);
		
		// compute control block address
		computeControlBlockAddr(models, cntlInfo);
		
		// get the symbol names
		fetchSymbolNames((INamedProcess[]) models);

	}

	/**
	 * Reads and fills the ReadyQueueData[]<br>
	 * Also, sets the control block addresses and
	 * the queue indices to the TaskData[].
	 *
	 * TaskData must be read before invoking this method.
	 */
	public void getRdyQData() {
		
		KernelObjectInfo[] objInfo = objTable.getObjectInfoTable();
		
		// these object infos will be used
		KernelObjectInfo cntlInfo = objInfo[ToppersObjectTable.TYPE_READY_QUEUE];
	
		// get data model
		AbstractDataModel[] models = input.getReadyQueueData();
		
		// read data
		readAndParse(cntlInfo, models);
		
		// compute control block address
		computeControlBlockAddr(models, cntlInfo);

		// compute queue list
		computeQueueList(models, input.getTaskData());
		
	}

	/**
	 * Reads and fills the CyclicHandlerData[]<br>
	 * Also, sets the control block addresses and 
	 * retreives the handler names for each handler. 
	 * 
	 */
	public void getCycData() {
		
		KernelObjectInfo[] objInfo = objTable.getObjectInfoTable();
	
		// these object infos will be used
		KernelObjectInfo cntlInfo = objInfo[ToppersObjectTable.TYPE_CYC_CTRL];
		KernelObjectInfo initInfo = objInfo[ToppersObjectTable.TYPE_CYC_INIT];
		
		// get data model
		AbstractDataModel[] models = input.getCyclicHandlerData();
		
		// read data
		readAndParse(cntlInfo, models);
		readAndParse(initInfo, models);
		
		// compute control block address
		computeControlBlockAddr(models, cntlInfo);

		// get the symbol names
		fetchSymbolNames((INamedProcess[]) models); 
		
	}
	
	/**
	 * Reads and fills the MailBoxData[]<br>
	 * Also, sets the control block addresses and
	 * the queue indices to the TaskData[].
	 *  
	 * TaskData must be read before invoking this method.
	 */
	public void getMbxData() {
		
		KernelObjectInfo[] objInfo = objTable.getObjectInfoTable();
		
		// these object infos will be used
		KernelObjectInfo cntlInfo = objInfo[ToppersObjectTable.TYPE_MAILBOX_CTRL];
		KernelObjectInfo initInfo = objInfo[ToppersObjectTable.TYPE_MAILBOX_INIT];
		
		// get data model
		AbstractDataModel[] models = input.getMailBoxData();
		
		// read data
		readAndParse(cntlInfo, models);
		readAndParse(initInfo, models);
		
		// compute control block address
		computeControlBlockAddr(models, cntlInfo);

		// compute queue list
		computeQueueList(models, input.getTaskData());
		
	}
	
	/**
	 * Reads and fills the SemaphoreData[]<br>
	 * Also, sets the control block addresses and
	 * the queue indices to the TaskData[].
	 *
	 * TaskData must be read before invoking this method.
	 */
	public void getSemData() {
		
		KernelObjectInfo[] objInfo = objTable.getObjectInfoTable();
		
		// these object infos will be used
		KernelObjectInfo cntlInfo = objInfo[ToppersObjectTable.TYPE_SEM_CTRL];
		KernelObjectInfo initInfo = objInfo[ToppersObjectTable.TYPE_SEM_INIT];
	
		// get data model
		AbstractDataModel[] models = input.getSemaphoreData();
		
		// read data
		readAndParse(cntlInfo, models);
		readAndParse(initInfo, models);
	
		// compute control block address
		computeControlBlockAddr(models, cntlInfo);

		// compute queue list
		computeQueueList(models, input.getTaskData());
		
	}
	
	/**
	 * Reads and fills the EventFlagData[]<br>
	 * Also, sets the control block addresses and 
	 * the queue indices to the TaskData[] and
	 * If the queue has a list, then reads the wait info for
	 * each task.
	 * 
	 * TaskData must be read before invoking this method.
	 */
	public void getFlgData() {
		
		KernelObjectInfo[] objInfo = objTable.getObjectInfoTable();
		
		// these object infos will be used
		KernelObjectInfo cntlInfo = objInfo[ToppersObjectTable.TYPE_EVENT_CTRL];
		KernelObjectInfo initInfo = objInfo[ToppersObjectTable.TYPE_EVENT_INIT];
	
		// get data model
		AbstractDataModel[] models = input.getEventFlagData();
		
		// read data
		readAndParse(cntlInfo, models);
		readAndParse(initInfo, models);
		
		// compute control block address
		computeControlBlockAddr(models, cntlInfo);

		// compute queue list
		computeQueueList(models, input.getTaskData());
		
		// get the wait info and set it to the TaskData
		readWaitInfo((IQueue[]) models, input.getTaskData());
		
	}
	
	/**
	 * Reads and fills the DataQueueData[]<br>
	 * Also, sets the control block addresses and
	 * the queue indices to the TaskData[].
	 *
	 * TaskData must be read before invoking this method.
	 */
	public void getDtqData() {
		
		KernelObjectInfo[] objInfo = objTable.getObjectInfoTable();
		
		// these object infos will be used
		KernelObjectInfo cntlInfo = objInfo[ToppersObjectTable.TYPE_DATA_QUEUE_CTRL];
		KernelObjectInfo initInfo = objInfo[ToppersObjectTable.TYPE_DATA_QUEUE_INIT];
	
		// get data model
		AbstractDataModel[] models = input.getDataQueueData();
		
		// read data
		readAndParse(cntlInfo, models);
		readAndParse(initInfo, models);
	
		// compute control block address
		computeControlBlockAddr(models, cntlInfo);

		// compute send/recv queue list
		TaskData[] tasks = input.getTaskData();
		for (int i = 0; i < models.length; i++) {
			
			// get the control block address
			long sendEndAddr = models[i].getControlBlockAddress();
			long recvEndAddr = models[i].getControlBlockAddress() + 12;

			IQueue queueObject = (IQueue) models[i];

			// get the next address of the send queue 
			long nextAddr = queueObject.getNextQueue(DataQueueData.SEND_WAIT_QUEUE_TYPE);
			// get indices for send queue
			int[] indices = getQueueListIndices(queueObject, DataQueueData.SEND_WAIT_QUEUE_TYPE, sendEndAddr, tasks);
			queueObject.setQueueIndices(indices, DataQueueData.SEND_WAIT_QUEUE_TYPE);
			
			
			// get the next address of receive queue
			nextAddr = queueObject.getNextQueue(DataQueueData.RECV_WAIT_QUEUE_TYPE);
			// get indices for receive queue
			indices = getQueueListIndices(queueObject, DataQueueData.RECV_WAIT_QUEUE_TYPE, recvEndAddr, tasks);
			queueObject.setQueueIndices(indices, DataQueueData.RECV_WAIT_QUEUE_TYPE);

		}
		
	}
	
	/**
	 * Reads and fills the CyclicHandlerData[]<br>
	 * Also, sets the control block addresses and
	 * the queue indices to the TaskData[].
	 *
	 * TaskData must be read before invoking this method.
	 */
	public void getMpfData() {
		
		KernelObjectInfo[] objInfo = objTable.getObjectInfoTable();
		
		// these object infos will be used
		KernelObjectInfo cntlInfo = objInfo[ToppersObjectTable.TYPE_FIX_MEM_CTRL];
		KernelObjectInfo initInfo = objInfo[ToppersObjectTable.TYPE_FIX_MEM_INIT];
	
		// get data model
		AbstractDataModel[] models = input.getFixedMemoryPoolData();
		
		// read data
		readAndParse(cntlInfo, models);
		readAndParse(initInfo, models);
	
		// compute control block address
		computeControlBlockAddr(models, cntlInfo);

		// compute queue list
		computeQueueList(models, input.getTaskData());
		
	}
	
	/**
	 * Reads and fills the CyclicHandlerData[]<br>
	 * Also, retreives the handler names for each handler. 
	 * 
	 * 
	 */
	public void getIntHdlData() {
		
		KernelObjectInfo[] objInfo = objTable.getObjectInfoTable();
		
		// these object infos will be used
		KernelObjectInfo initInfo = objInfo[ToppersObjectTable.TYPE_INTERRUPT_INIT];
		
		// get data model
		AbstractDataModel[] models = input.getInterruptHandlerData();
		
		// read data
		readAndParse(initInfo, models);

		// get the symbol names
		fetchSymbolNames((INamedProcess[]) models);
		
	}

	/**
	 * Reads and fills the CyclicHandlerData[]<br>
	 * Also, retreives the handler names for each handler. 
	 *
	 */
	public void getExpHdlData() {
		
		KernelObjectInfo[] objInfo = objTable.getObjectInfoTable();
		
		// these object infos will be used
		KernelObjectInfo initInfo = objInfo[ToppersObjectTable.TYPE_EXCEPTION_INIT];
		
		// get data model
		AbstractDataModel[] models = input.getExceptionHandlerData();
		
		// read data
		readAndParse(initInfo, models);
		
		// get the symbol names
		fetchSymbolNames((INamedProcess[]) models);
		
		
	}
	

	
	
	/**
	 * Computes the control block addresses for the given models, 
	 * based on the base address and size stored in KernelObjectInfo.<br>
	 * 
	 * AbstractDataModel array should be filled before invoking this method.
	 * 
	 * @param models		the models that need their address computed.
	 * @param objInfo		info of the base address and data size.
	 */
	private void computeControlBlockAddr(AbstractDataModel[] models, KernelObjectInfo objInfo) {
		
		long baseAddress = objInfo.dataAddress;
		long size = objInfo.dataSize;
		
		for (int i = 0; i < models.length; i++) {
			models[i].controlBlockAddress = baseAddress + size * i;  
		}
	
	}
	
	
	/**
	 * Calls getQueueListIndices() for each AbstractDataModel and
	 * sets the indices to them.<br>
	 * 
	 * Easy access method for retreiving IQueue.DEFAULT_TYPE queues.
	 * Cannot be used for DataQueueData.
	 * 
	 * TaskData array should be filled before invoking this method.
	 * 
	 * @param models		AbstractDataModel array (which implements IQueue)
	 * @param taskDatas	the TaskData
	 */
	private void computeQueueList(AbstractDataModel models[], TaskData[] taskDatas) {
		
		for (int i = 0; i < models.length; i++) {
			IQueue primeQueue = (IQueue) models[i];
			int[] indices = getQueueListIndices(primeQueue, IQueue.DEFAULT_TYPE, models[i].getControlBlockAddress(), taskDatas);
			primeQueue.setQueueIndices(indices, IQueue.DEFAULT_TYPE);
		}
	
	} 
	
	
	/**
	 * Returns the indices of the TaskData array which indicate 
	 * a queue chain.
	 * The starting point will be given as IQueue(first parameter). 
	 * After this method is invoked, search for the TaskData array
	 * to form a queue list.
	 * 
	 * @param initialQ		starting point to search the queue.
	 * @param queueType	
	 * @param endAddr		address indicating where the searching should end. 
	 * @param taskDatas	array of TaskData. 
	 * 						These data should have their control block addresses.
	 * @return indices of the TaskData array which represents the queue chain.
	 */
	private int[] getQueueListIndices(IQueue initialQ, int queueType, long endAddr, TaskData[] taskDatas) {

		List indexList = new ArrayList();
		
		// get the next task data's address
		long next = initialQ.getNextQueue(queueType);
		while (next != endAddr) {
			int taskIndex = -1;
			
			// get the task's index where "next" is identical to 
			for (int i = 0; i < taskDatas.length; i++) {
				if (taskDatas[i].getControlBlockAddress() == next) {
					taskIndex = i;
					break;
				}
			}
		
			if (taskIndex < 0) {
				break;	// should never occur
			}
			
			// set the index to the list
			indexList.add(new Integer(taskIndex));
			
			// get the next task task data's address
			next = taskDatas[taskIndex].getNextQueue(IQueue.DEFAULT_TYPE);
	
		}
		
		// make the return value as an int array
		int[] indices = new int[indexList.size()];
		for (int i = 0; i < indexList.size(); i++) {
			indices[i] = ((Integer) indexList.get(i)).intValue();
		}
		
		// clear the list to release references
		indexList.clear();
	
		return indices;
	
	}
	
	
	/**
	 * Reads the symbol and sets it to the given INamedProcess. 
	 * @param process INamedProcess object which will return the address for the key.
	 */
	private void fetchSymbolNames(INamedProcess[] process) {
		
		for (int i = 0; i < process.length; i++) {
			String name = readSymbolName(process[i].getAddress());
			process[i].setProcessName(name);
		}
	
	}
	
	
	/**
	 * Returns the index(0 origin task ID) of TaskData array whose control block
	 * address is identical with the first argument.
	 * 
	 * @param address		the control block address 
	 * @param tasks		the TaskData array to be searched from
	 * @return 	the index. returns -1 if the address is 0, returns -2 if unidentified.
	 */
	public int getTaskIdFromAddress(long address, TaskData[] tasks) {
		
		int ret = -2;
		
		if (address == 0x0L) {
			return -1;
		}
		
		for (int i = 0; i < tasks.length; i++) {
			if (address == tasks[i].getControlBlockAddress()) {
				ret = i;
				break;
			}
		}
		
		return ret;
		
	}

	/**
	 * Reset all the counters so the next
	 * refresh will be performed.
	 * 
	 */
	private void resetCounters() {
		
		// initialize the requestCounters to -1
		for (int i = 0; i < requestCounters.length; i++) {
			requestCounters[i] = -1;
		}

	}
	

	/**
	 * Do the actual event handling
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.IOSMonitorController#handleRequest(int, jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.command.MonitorCommand)
	 * 
	 * @param	tabIndex	the 0-related index of the tab  
	 * @param	command		the command to be executed. see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.command.MonitorCommand
	 */
	public void handleRequest(int tabIndex, MonitorCommand command) {
	
		// check the request counter
		int currentCount = readCounter();
		int previousCount = requestCounters[tabIndex];
		requestCounters[tabIndex] = currentCount;
		if (currentCount == previousCount) {
			if (OSMonitorPlugin.getDefault().isDebugging()) {
				System.out.println("Skipped data reading");
			}
			return;
		}
		
		// get the sysInfo
		readFromSymbol(ToppersSystemInfo.ADDRESS_SYMBOL, ToppersSystemInfo.READ_SIZE, sysInfo);
		if (sysInfo.isLoaded()) {
			input.setReadyQueuePriority(sysInfo.getMinPriority());
		}
		
		// execute it!
		command.executeCommand();
	
	} 

}
